package cologne.eck.all_peas.cloud.gui;

/*
 * PeaFactory - Production of Password Encryption Archives
 * Copyright (C) 2020  Axel von dem Bruch
 * 
 * This library is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU General Public License as published 
 * by the Free Software Foundation; either version 2 of the License, 
 * or (at your option) any later version.
 * This library is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * See:  http://www.gnu.org/licenses/gpl-2.0.html
 * You should have received a copy of the GNU General Public License 
 * along with this library.
 */

import java.awt.Font;
import java.awt.Window;
import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.IOException;
import java.lang.invoke.MethodHandles;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.ResourceBundle;

import javax.accessibility.AccessibleContext;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.SwingUtilities;

import cologne.eck.all_peas.cloud.CloudControl;
import cologne.eck.all_peas.cloud.CloudPropertyHandler;
import cologne.eck.all_peas.cloud.StorageProvider;
import cologne.eck.all_peas.cloud.provider.WebDavProvider;
import cologne.eck.all_peas.control.PeaControl;
import cologne.eck.all_peas.data.AttachmentHandler;
import cologne.eck.all_peas.data.PeaProperties;
import cologne.eck.all_peas.files.ExecutionTimeObserver;
import cologne.eck.all_peas.files.FileComposer;
import cologne.eck.all_peas.files.FilePanelHolder;
import cologne.eck.all_peas.files.FileTypePanel;
import cologne.eck.all_peas.gui.CurrentFileDisplay;
import cologne.eck.all_peas.gui.CursorManager;
import cologne.eck.all_peas.gui.HtmlFileViewer;
import cologne.eck.all_peas.gui.PeaDialog;
import cologne.eck.all_peas.gui.PeaFileChooser;
import cologne.eck.all_peas.gui.PeaLockFrame;
import cologne.eck.all_peas.gui.PswDialogView;
import cologne.eck.all_peas.vm_specific.JREProperties;
import cologne.eck.peafactory.crypto.CipherStuff;
import cologne.eck.peafactory.crypto.kdf.KeyDerivation;
import cologne.eck.tools.Converter;
import cologne.eck.tools.ExceptionHelper;
import cologne.eck.tools.FileTools;
import cologne.eck.tools.TestLog;
import cologne.eck.tools.UnexpectedValueException;
import cologne.eck.tools.Zeroizer;

@SuppressWarnings("serial")
public class CloudMenu extends JMenu implements ActionListener {
	
	private ResourceBundle languagesBundle;

	private Font font;
	
	private PeaLockFrame lockFrame;
	
	private Window rootWindow;
	
	private JMenu saveMenu;// to fire programmatically sva action
	
//	private SwingWorker<Void, Void> sw = null;
	
//	private static DateTimeFormatter formatter = DateTimeFormatter.ofLocalizedDateTime(FormatStyle.MEDIUM, FormatStyle.MEDIUM);//DateTimeFormatter.ISO_LOCAL_DATE_TIME;

	/**
	 * Menu for cloud actions
	 * 
	 * @param _window	the window that holds the menu
	 */
	public CloudMenu(Window _window) {
		
		rootWindow = _window;
		
		font = new Font(Font.SANS_SERIF, Font.PLAIN, PeaProperties.getFontSize());
		
		languagesBundle = (ResourceBundle) JREProperties.getBundle();
		this.setFont(font);
		this.setText(languagesBundle.getString("cloud"));
		this.setMnemonic(languagesBundle.getString("cloud").charAt(0));

		addItems();
	}
	
	private void addItems() {
		String[] providers = CloudPropertyHandler.getProviderNames();
		if (providers != null) {
			// save and update menu only for lockFrame
			if ( rootWindow == null || rootWindow.getName() == null ||  ! rootWindow.getName().equals("EncryptedDialog")) {
				JMenu downloadMenu = new JMenu(languagesBundle.getString("download_from_cloud"));
				downloadMenu.setFont(font);
				downloadMenu.setToolTipText(languagesBundle.getString("tooltip_cloud_download"));
				downloadMenu.setMnemonic('D');
				downloadMenu.addActionListener(this);
				this.add(downloadMenu);
				int len = providers.length;
				for (int i = 0; i < len; i++) {
					JMenuItem downloadProviderItem = new JMenuItem(providers[i]);
					downloadProviderItem.getAccessibleContext().
					setAccessibleDescription(
							"Download files from " + providers[i]);
					downloadProviderItem.setActionCommand("download_" + providers[i]);
					downloadProviderItem.setFont(font);
					//webItem.setMnemonic(KeyEvent.VK_I);
					downloadProviderItem.setMnemonic(providers[i].charAt(0));
					downloadProviderItem.addActionListener(this);
					downloadMenu.add(downloadProviderItem);
				}
				
				/*JMenu updateMenu = new JMenu(languagesBundle.getString("look_for_cloud_update"));
				updateMenu.setFont(font);
				updateMenu.setToolTipText(languagesBundle.getString("tooltip_look_update"));
				updateMenu.setMnemonic('U');
				updateMenu.addActionListener(this);
				this.add(updateMenu);
				
				int len = providers.length;
				for (int i = 0; i < len; i++) {
					
			/*		JMenuItem updateProviderItem = new JMenuItem(providers[i]);
					updateProviderItem.getAccessibleContext().
					setAccessibleDescription(
							"Look for updates in " + providers[i]);
					updateProviderItem.setActionCommand("update_" + providers[i]);
					updateProviderItem.setFont(font);
					//webItem.setMnemonic(KeyEvent.VK_I);
					updateProviderItem.setMnemonic(providers[i].charAt(0));
					updateProviderItem.addActionListener(this);
					updateMenu.add(updateProviderItem);
				}
				*/

				saveMenu = new JMenu(languagesBundle.getString("save_in_cloud"));
				saveMenu.setFont(font);
				//saveMenu.setToolTipText(languagesBundle.getString("tooltip_look_update"));
				saveMenu.setMnemonic('S');
				//treeViewItem.setMnemonic(KeyStroke.getKeyStroke(languagesBundle.getString("instruction_mnemonic")).getKeyCode());
				saveMenu.addActionListener(this);
				this.add(saveMenu);


				for (int i = 0; i < len; i++) {
					JMenuItem updateProviderItem = new JMenuItem(providers[i]);
					updateProviderItem.getAccessibleContext().
					setAccessibleDescription(
							"Saves current file in " + providers[i]);
					updateProviderItem.setActionCommand("save_" + providers[i]);
					updateProviderItem.setFont(font);
					//webItem.setMnemonic(KeyEvent.VK_I);
					updateProviderItem.setMnemonic(providers[i].charAt(0));
					updateProviderItem.addActionListener(this);
					saveMenu.add(updateProviderItem);
				}
			} else { // this is in encryptedWindow (PswDialogView)
				
				JMenu openMenu = new JMenu(languagesBundle.getString("open_file"));
				openMenu.setFont(font);
				//updateMenu.setToolTipText(languagesBundle.getString("tooltip_look_update"));
				openMenu.setMnemonic('O');
				openMenu.addActionListener(this);				
				this.add(openMenu);
				int len = providers.length;
				for (int i = 0; i < len; i++) {
					JMenuItem openProviderItem = new JMenuItem(providers[i]);
					openProviderItem.getAccessibleContext().
					setAccessibleDescription(
							"Look for files in " + providers[i]);
					openProviderItem.setActionCommand("open_" + providers[i]);
					openProviderItem.setFont(font);
					//webItem.setMnemonic(KeyEvent.VK_I);
					openProviderItem.setMnemonic(providers[i].charAt(0));
					openProviderItem.addActionListener(this);
					openMenu.add(openProviderItem);
				}
			}
		}
		
		JMenuItem createNewItem = new JMenuItem(languagesBundle.getString("add_cloud_provider"));
		createNewItem.getAccessibleContext().
	    setAccessibleDescription(
	    	    "Create a new cloud access for one provider.");
		createNewItem.setActionCommand("addProvider");
		createNewItem.setFont(font);
		createNewItem.addActionListener(this);
		this.add(createNewItem);

		if (providers != null) {
			JMenu modifyMenu = new JMenu(languagesBundle.getString("modify_cloud_provider"));
			modifyMenu.getAccessibleContext().
			setAccessibleDescription(
					"Modify existing provider.");
			modifyMenu.setFont(font);
			int len = providers.length;
			for (int i = 0; i < len; i++) {

				JMenuItem modifyItem = new JMenuItem(providers[i]);
				modifyItem.getAccessibleContext().
				setAccessibleDescription(
						"Modify provider " + providers[i]);
				modifyItem.setActionCommand("modify_" + providers[i]);
				modifyItem.setFont(font);
				modifyItem.setMnemonic(providers[i].charAt(0));
				modifyItem.addActionListener(this);
				modifyMenu.add(modifyItem);
			}
			this.add(modifyMenu);
		}
		if ( rootWindow != null && rootWindow.getName().equals("EncryptedDialog")) {
			// TODO also for lockFrame: update view in lockFrame
			if (providers != null) {
				JMenu deleteMenu = new JMenu(languagesBundle.getString("delete_cloud_provider"));
				deleteMenu.getAccessibleContext().
				setAccessibleDescription(
						"Delete existing provider.");
				deleteMenu.setFont(font);
				int len = providers.length;
				for (int i = 0; i < len; i++) {

					JMenuItem deleteItem = new JMenuItem(providers[i]);
					deleteItem.getAccessibleContext().
					setAccessibleDescription(
							"Delete provider " + providers[i]);
					deleteItem.setActionCommand("delete_" + providers[i]);
					deleteItem.setFont(font);
					deleteItem.setMnemonic(providers[i].charAt(0));				
					deleteItem.addActionListener(this);
					deleteMenu.add(deleteItem);
				}
				this.add(deleteMenu);
			}
		}
	}
	/**
	 * Rebuild the CloudMenu.
	 * This should be called whenever aprovider was added or deleted
	 */
	private void updateView() {
		this.removeAll();
		this.addItems();
		this.getParent().validate();
		this.getParent().repaint();
		//this.updateUI();
	}
	
	protected static void displayMessage(Component owner, String message, boolean isError) {
		PeaLockFrame lockFrame = null;
		if (owner instanceof PeaLockFrame) {
			lockFrame = (PeaLockFrame) owner;
			lockFrame.setMessage(message, isError);
		} else if (owner instanceof PswDialogView) {
			PswDialogView.setMessage(message, isError, false);
		} else {
			new UnexpectedValueException("Component", "owner", 
					"is not instance of PeaLockFrame or PswDialogView").printDescription();
			//return PeaProperties.getOsBridge().translate("unexpected_error");
		}
	}
	/**
	 * Show login dialog for provider, fill with properties, 
	 * set properties, store password
	 * 
	 * @param rootWindow	PswDialogView or PeaLockFrame
	 * @param title			the title to show
	 * @param provider		the name of the provider
	 * 
	 * @return	true if properties were set, false otherwise (cancelled or error)
	 */
	private boolean showLoginDialogAndSetProperties(Window rootWindow, String title, String provider) {
		String[] params = CloudPropertyHandler.getProviderLoginParameters(provider);
		char[] psw = null;
		if (! rootWindow.getName().equals("EncryptedDialog") && ! PeaProperties.getFileType().equals("file")) {// PeaLockFrame
			//	CloudPropertyHandler.setProviderSaltAndAlgos(provider);
			if (PeaProperties.getAlgoParamString() == null || KeyDerivation.getSalt() == null || provider == null) {
				TestLog.ve(getClass(), "Missing value for " + provider, 5);
				return false;
			}
			// get password from properties
			String pswString = CloudPropertyHandler.getPlainPasswordFromProvider(provider,
						Converter.bytes2hex(KeyDerivation.getSalt()), 
						Converter.bytes2hex(PeaProperties.getAlgoParamString()));
			if (pswString != null) {
				psw = pswString.toCharArray();
			} 
		} else {
			// get temporary encrypted password
			HashMap<String, byte[]> map = WebDavLogin.getPasswordsFromNewProviderDialog();
			if (map != null) {
				psw = Converter.bytes2chars(WebDavLogin.getTemporaryStoredPassword(provider));
			}
		}
		// show cloud login dialog
		WebDavLogin login = new WebDavLogin(
				rootWindow, 
				title,
				null, 
				provider, 
				params[0], 
				psw, 
				params[2]);
		int result = login.showLoginDialog();
		if (result == 0) {
			TestLog.v(this.getClass(), provider + " successfully modified");
			if (! rootWindow.getName().equals("EncryptedDialog")  && ! PeaProperties.getFileType().equals("file")) {// PeaLockFrame
				if (psw == null) {
					// was probably set in WebDavLogin:
					psw = CloudPropertyHandler.getPlainPasswordFromProvider(provider,
							Converter.bytes2hex(KeyDerivation.getSalt()), 
							Converter.bytes2hex(PeaProperties.getAlgoParamString())).toCharArray();
					if (psw == null) {
						// stored as manually typed password?
						psw = Converter.bytes2chars(WebDavLogin.getTemporaryStoredPassword(provider));
						if (psw == null) {
							// show password dialog
							psw = PeaDialog.showPasswordDialog(this, null, provider, false);
						}
						if (psw != null) {
							CloudPropertyHandler.storeEncryptedPasswordFromProvider(
									CloudPropertyHandler.encryptPassword(new String(psw)), 
									provider, Converter.bytes2hex(KeyDerivation.getSalt()), 
									Converter.bytes2hex(PeaProperties.getAlgoParamString()), null);							
						}
					} 
				}
				CloudPropertyHandler.createSubSet(provider,
						Converter.bytes2hex(KeyDerivation.getSalt()), 
						Converter.bytes2hex(PeaProperties.getAlgoParamString()),
						new String(psw), null);
			} else {
				// password is stored in WebDavLogin
			}
			return true;
		} else {
			if (result == 1) {
				// failed
				TestLog.e(getClass(), "Could not set properties for " + provider);
				TestLog.ve(getClass(), "Error occurred for " + provider, 5);
			}
			return false;
		}
	}
	
	
	@Override
	public void actionPerformed(ActionEvent ae) {
		String command = ae.getActionCommand();
		//TestLog.v(this.getClass(), "Command: " + command);
		
		Object lockObject = PeaControl.getDialog().getLockFrame();
		if (lockObject != null) {
			if (lockObject instanceof PeaLockFrame) {
				lockFrame = (PeaLockFrame) lockObject;
			}
		} 

		// Reset message label
		if (lockFrame != null) {
			lockFrame.setMessage("", false);
		} else {
			PswDialogView.setMessage("", false,false);
		}

		if (command.equals("addProvider")) {			
			if (new File (CloudPropertyHandler.getCloudPropertyFileName()).exists() == false) {
				HtmlFileViewer hfv = new HtmlFileViewer("cloud_guide", SwingUtilities.getWindowAncestor(this));
				hfv.requestFocus();
			}
			WebDavLogin login = new WebDavLogin(
					rootWindow, 
					languagesBundle.getString("add_cloud_provider"),
					null, 
					null, 
					null, 
					null, 
					null);
			int result = login.showLoginDialog();
			if (result == 0) {
				if (rootWindow.getName().equals("EncryptedDialog")) {// PswDialogView
					// open:
					StorageProvider prov = login.getProvider();
					String[] fileNames = prov.getCloudFiles(PeaControl.getDialog().getSupportedExtensions());
					//	( (WebDavProvider) prov).setWebDavBasePath(params[2]);
					CloudFileManager cfm = new CloudFileManager(rootWindow, prov, fileNames);
					cfm.setVisible(true);
					String[] selectedFileNames = cfm.getSelectedFileNames();
					if (PeaProperties.getFileType().equals("file")) {

						if (selectedFileNames == null) {
							return;
						}
						// download the encrypted files and add to FileTypePanel

						// choose a local directory to store the files from cloud:
						// open file chooser to select a directory to store file
						PeaFileChooser pfc = new PeaFileChooser(this);
						pfc.customizeFileChooser(languagesBundle.getString("save"), 
								languagesBundle.getString("select_folder_to_store_cloud_files"), false);
						pfc.setApproveButtonText(languagesBundle.getString("select"));
						pfc.setMultiSelectionEnabled(false);
						pfc.setFileSelectionMode(PeaFileChooser.DIRECTORIES_ONLY);
						pfc.showOpenDialog(this);
						// get selected directory
						File directory = pfc.getSelectedFile();
						if (directory == null) {
							TestLog.e(MethodHandles.lookup().lookupClass(), "No directory selected to store files from cloud...");
							return;
						}
						// create an array of file names with path (to ask to overwrite)
						String[] selectedFileNamesWithPath = new String[selectedFileNames.length];
						for (int i = 0; i < selectedFileNames.length; i++) {
							selectedFileNamesWithPath[i] = directory.getAbsoluteFile() + File.separator + selectedFileNames[i];
						}
						// ask to overwrite, if file exist:
						selectedFileNamesWithPath = FileTools.askToOverwriteFiles(selectedFileNamesWithPath);

						if (selectedFileNamesWithPath == null) {
							TestLog.e(CloudMenu.class, "No files to open left...");							
							return;
						}
						// create array of file names without path (to download)
						selectedFileNames = new String[selectedFileNamesWithPath.length];
						for (int i = 0; i < selectedFileNamesWithPath.length; i++) {
							selectedFileNames[i] = new File(selectedFileNamesWithPath[i]).getName();
						}
						// download the files in the chosen directory:
						String downloadError = prov.downloadFiles(selectedFileNames, directory.getAbsolutePath());
						if (downloadError != null) {
							PeaDialog.showMessage(rootWindow, downloadError, 
									PeaProperties.getVmBridge().translate("error"), 0);
						}
						// add files to FileTypePanel
						FileTypePanel ftp = (FileTypePanel)JREProperties.getTypePanel();
						FileComposer fc = ftp.getFileComposer();
						boolean fileAdded = false;
						for (String selectedFileName : selectedFileNames) {
							File f = new File(directory.getAbsoluteFile() + File.separator + selectedFileName);
							if (f.exists()) {
								// add to model an view:
								fc.addOrUpdateFile(f, false);
								fileAdded = true;
							} else {
								TestLog.e(CloudMenu.class, "Does not exist: " + f.getAbsolutePath());
								new UnexpectedValueException("File", "f", "does not exist: "+f.getAbsolutePath()).printDescription();
							}
						}
						// update view of FileTypePanel if check box was added:
						if (fileAdded == true) {
							ftp.updateFileView();
						}
					} else { // Notebook PEA, Calendar Lock PEA, Image Lock PEA

						// download the files in the chosen directory:
						String downloadError = prov.downloadFiles(selectedFileNames, CloudControl.getTmpDir()
								 + prov.getProviderName());
						if (downloadError != null) {
							PeaDialog.showMessage(rootWindow, downloadError, 
									PeaProperties.getVmBridge().translate("error"), 0);
						}

						// distribute and add to panel:
						int filesLen = selectedFileNames.length;
						File[] files = new File[filesLen];
						String[] selectedFileNamesWithPath = new String[filesLen];
						for (int i = 0; i < filesLen; i++) {							
							selectedFileNamesWithPath[i] = CloudControl.getTmpFileNameForCloudFile(selectedFileNames[i], prov.getProviderName(), null);
							files[i] = new File(selectedFileNamesWithPath[i]);
						}
						// register that these files are already downloaded:
						CloudControl.addDownloadedInitialFileNames(selectedFileNamesWithPath);
						boolean fileAdded = AttachmentHandler.distributeFiles(files);
				    	if (fileAdded == true) {
				    		// update view:
				    		FilePanelHolder.updateWindow();
				    	}
					}
				}

				this.updateView();
			} else {
				if (result == 1) {
					TestLog.e(CloudMenu.class, "Login failed...");
				} else if (result == -1) {
					TestLog.v(CloudMenu.class, "Login cancelled...");
				}
			}

		} else if (command.startsWith("modify_")) {		
			String provider = command.substring("modify_".length(), command.length());
			//TestLog.v(this.getClass(), "Modify " + provider);
			boolean success = showLoginDialogAndSetProperties(rootWindow, languagesBundle.getString("modify_cloud_provider"), provider);
			if (success == true) {
				TestLog.v(getClass(), "properties set for " + provider);
			}
		    
		} else if (command.startsWith("delete_")) {
			String provider = command.substring("delete_".length(), command.length());
			//TestLog.v(this.getClass(), "Delete " + provider);
			int result = PeaDialog.showQuestion(this.getRootPane(), 
					languagesBundle.getString("really_delete")
					+ "\n" + provider + "?");
			if (result == 0) {
				CloudPropertyHandler.deleteProviderParameters(provider);
				//  remove files from CloudFilePanel and model if in PswDialogView
				if (lockObject == null) {
					ArrayList<FileTypePanel> ftpList = FilePanelHolder.getFileTypePanelList();
					if (ftpList != null &&  ! ftpList.isEmpty()) {
						for (FileTypePanel ftp : ftpList) {
							ftp.getCloudFilePanel().getCloudControl().removeInitialProvider(provider);
						}
					}
				} else {
					((FileTypePanel)JREProperties.getTypePanel()).getCloudFilePanel().getCloudControl().removeUsedProvider(provider);
				}
				this.updateView();
				// update view:
				//FilePanelHolder.updateFileView();
				FilePanelHolder.updateWindow();
			}			
		} else if (command.startsWith("open_")) {
			/*
			 * This only used in PswDialogView: download an encrypted file from a provider, 
			 * distribute depending on salt and algorithms/parameters
			 * and store in tmp directory or chosen directory (File Lock PEA)
			 */
			// Open a provider manually, when it was not opened automatically
			// ("open_" + providers[i])
			String provider = command.substring("open_".length(), command.length());
			TestLog.v(this.getClass().getSimpleName(), "Open: " + provider);
		
			String[] params = CloudPropertyHandler.getProviderLoginParameters(provider);
			StorageProvider prov = null;

			if (params[0] == null || params[2] == null) { // user name, WebDAV path - this should not happen...
				TestLog.e(getClass(), "Missing user name or WebDAV base path", 5);
				boolean success = showLoginDialogAndSetProperties(rootWindow, languagesBundle.getString("modify_cloud_provider"), provider);
				if (success == true) {
					TestLog.v(getClass(), "properties set for " + provider);
				}
			}
			// get the password: Check if it was stored from adding a new provider
			byte[] pswBytes = null;
			char[] psw = null;
			if (WebDavLogin.getPasswordsFromNewProviderDialog() != null) {
				pswBytes = WebDavLogin.getPasswordsFromNewProviderDialog().get(provider);
			}
			if (pswBytes != null) {
				psw = Converter.bytes2chars(WebDavLogin.getPasswordsFromNewProviderDialog().get(provider));
			} else {
				psw = PeaDialog.showPasswordDialog(this, provider + ": " + languagesBundle.getString("enter_cloud_password"), 
						languagesBundle.getString("password"), false);
			}
			// set wait cursor:
			if (SwingUtilities.getWindowAncestor(this) != null) {
				SwingUtilities.getWindowAncestor(this).setCursor(CursorManager.getWaitCursor());
			}
			StorageProvider sp = CloudControl.getOpenProvider(provider);
			if (sp == null) {
				 sp = new WebDavProvider(provider);
			}
			if (sp instanceof WebDavProvider) {
				prov = (WebDavProvider) sp;
			}
			//prov = 	 new WebDavProvider(provider);
			( (WebDavProvider) prov).setWebDavBasePath(params[2]);
			byte[] plainPswBytes = Converter.chars2bytes(psw);
			String error = prov.authorize(params[0], psw, null);
			if (error == null) {
				// store password encrypted:
				WebDavLogin.storePassword(provider, plainPswBytes);
				Zeroizer.zero(plainPswBytes);
				// get file names:
				String[] fileNames = prov.getCloudFiles(PeaControl.getDialog().getSupportedExtensions());
				( (WebDavProvider) prov).setWebDavBasePath(params[2]);
				CloudFileManager cfm = new CloudFileManager(rootWindow, prov, fileNames);
				// reset cursor
				if (SwingUtilities.getWindowAncestor(this) != null) {
					SwingUtilities.getWindowAncestor(this).setCursor(CursorManager.getDefaultCursor());
				}
				cfm.setVisible(true);
				String[] selectedFileNames = cfm.getSelectedFileNames();
				if (selectedFileNames != null && selectedFileNames.length > 0) {
					TestLog.v(this.getClass().getSimpleName(), "Number of selected files in cloud: " + selectedFileNames.length);
				}
				if (selectedFileNames != null) {
					// create an array of file names with path (to ask to overwrite)
					String[] selectedFileNamesWithPath = new String[selectedFileNames.length];
					File directory = null;
					if (PeaProperties.getFileType().equals("file")) {
						// download the encrypted files and add to FileTypePanel
						
						// choose a local directory to store the files from cloud:
						// open file chooser to select a directory to store file
						PeaFileChooser pfc = new PeaFileChooser(this);
						pfc.customizeFileChooser(languagesBundle.getString("save"), 
								languagesBundle.getString("select_folder_to_store_cloud_files"), false);
						pfc.setApproveButtonText(languagesBundle.getString("select"));
						pfc.setMultiSelectionEnabled(false);
						pfc.setFileSelectionMode(PeaFileChooser.DIRECTORIES_ONLY);
						pfc.showOpenDialog(this);
						// get selected directory
						directory = pfc.getSelectedFile();
						if (directory == null) {
							TestLog.e(CloudMenu.class, "No directory selected to store files from cloud...");
							return;
						}

						for (int i = 0; i < selectedFileNames.length; i++) {
							selectedFileNamesWithPath[i] = directory.getAbsoluteFile() + File.separator + selectedFileNames[i];
						}
						// ask to overwrite, if file exist:
						selectedFileNamesWithPath = FileTools.askToOverwriteFiles(selectedFileNamesWithPath);
					} else { // all other PEAs except File Lock PEA
						// create tmp directory
						directory = new File(CloudControl.getTmpDir() + provider);
						if ( ! directory.exists()){
							directory.mkdirs();
						}
						for (int i = 0; i < selectedFileNames.length; i++) {
							selectedFileNamesWithPath[i] = CloudControl.getTmpFileNameForCloudFile(selectedFileNames[i], 
									provider, null);
									//directory.getAbsoluteFile() + File.separator + selectedFileNames[i];
						}
					}
						
						if (selectedFileNamesWithPath == null) {
							TestLog.e(CloudMenu.class, "No files to open left...");							
							return;
						}
						// create array of file names without path (to download)
						// TODO test if necessary...
		/*				selectedFileNames = new String[selectedFileNamesWithPath.length];
						for (int i = 0; i < selectedFileNamesWithPath.length; i++) {
							selectedFileNames[i] = new File(selectedFileNamesWithPath[i]).getName();
						}*/
						TestLog.v(this.getClass().getSimpleName(), "Number of files to download: " + selectedFileNames.length);
						// download the files in the chosen directory:
						String downloadError = prov.downloadFiles(selectedFileNames, directory.getAbsolutePath());
						if (downloadError != null) {
							PeaDialog.showMessage(rootWindow, downloadError, 
									PeaProperties.getVmBridge().translate("error"), 0);
						}
						// distribute and add to panel:
						int filesLen = selectedFileNames.length;
						File[] files = new File[filesLen];
						// possible errors, maybe not all files are still present
					//	selectedFileNamesWithPath = new String[filesLen];
						for (int i = 0; i < filesLen; i++) {
							files[i] = new File(selectedFileNamesWithPath[i]);
						//	selectedFileNamesWithPath[i] = CloudControl.getTmpFileNameForCloudFile(selectedFileNames[i], prov.getProviderName(), null);
						}
						// register that these files are already downloaded:
						if (! PeaProperties.getFileType().equals("file")) {
							CloudControl.addDownloadedInitialFileNames(selectedFileNamesWithPath);
						}
						boolean fileAdded = AttachmentHandler.distributeFiles(files);
				    	if (fileAdded == true) {
				    		// update view:
				    		//FilePanelHolder.updateFileView();
				    		FilePanelHolder.updateWindow();
				    	}
				}
			} else {
				Zeroizer.zero(plainPswBytes);
				if (SwingUtilities.getWindowAncestor(this) != null) {
					SwingUtilities.getWindowAncestor(this).setCursor(CursorManager.getDefaultCursor());
				}
				PeaDialog.showMessage(this, languagesBundle.getString("authorization_failed")
						+ "\n" + error, languagesBundle.getString("error"), 0);
			}


		} else if (command.startsWith("download_")) { // open in lockFrame
			/*
			 * This is used in a PeaLockFrame to download and decrypt file (load from...)
			 * and store in tmp directory or a chosen directory (File Lock PEA)
			 */

			String directoryName = null;

			if (PeaProperties.getFileType().equals("file")) {
				// download the encrypted files and add to FileTypePanel
				// choose a local directory to store the files from cloud:
				// open file chooser to select a directory to store file
				PeaFileChooser pfc = new PeaFileChooser(this);
				pfc.customizeFileChooser(languagesBundle.getString("save"), 
						languagesBundle.getString("select_folder_to_store_cloud_files"), false);
				pfc.setApproveButtonText(languagesBundle.getString("select"));
				pfc.setMultiSelectionEnabled(false);
				pfc.setFileSelectionMode(PeaFileChooser.DIRECTORIES_ONLY);
				pfc.showOpenDialog(this);
				// get selected directory
				File directory = pfc.getSelectedFile();
				if (directory == null) {
					TestLog.e(CloudMenu.class, "No directory selected to store files from cloud...");
					return;
				}
				directoryName = directory.getAbsolutePath();
			}

			String providerName = command.substring("download_".length(), command.length());
			
			String[] params =  CloudPropertyHandler.getProviderLoginParameters(providerName);
			String pswString = CloudPropertyHandler.getPlainPasswordFromProvider(providerName, 
					Converter.bytes2hex(KeyDerivation.getSalt()), 
					Converter.bytes2hex(PeaProperties.getAlgoParamString()));
			
			if (params[0] == null || params[2] == null ) {// no user or no WebDav path
				// set new parameters along with password and store as properties
				boolean success = showLoginDialogAndSetProperties(rootWindow, languagesBundle.getString("modify_cloud_provider"), providerName);
				if (success == true) {
					TestLog.v(getClass(), "properties set for " + providerName);
				} else {
					TestLog.e(getClass(), "could not set properties set for " + providerName);
					TestLog.ve(getClass(), "could not set properties set for " + providerName, 5);
					return;
				}
			} else if (pswString == null) { // only password not set
				byte[] pswBytes = WebDavLogin.getTemporaryStoredPassword(providerName);
				if (pswBytes != null) {
					pswString = new String(pswBytes, PeaProperties.getCharset());
				} else {
					char[] psw = PeaDialog.showPasswordDialog(this, null, providerName, false);
					if (psw != null) {
						pswString = new String(psw);
					} else { 
						TestLog.e(getClass(), "could not get password for " + providerName);
						TestLog.ve(getClass(), "could not get password for " + providerName, 5);
						return;
					}
				}
				if (pswString != null) { // store password
					CloudPropertyHandler.storeEncryptedPasswordFromProvider(
							CloudPropertyHandler.encryptPassword(pswString), 
							providerName, Converter.bytes2hex(KeyDerivation.getSalt()), 
							Converter.bytes2hex(PeaProperties.getAlgoParamString()), null);			
				}
			}

			StorageProvider sp = CloudControl.getOpenProvider(providerName);
			if (sp == null) {
				sp = new WebDavProvider(providerName);
			}
			WebDavProvider prov = null;
			if (sp instanceof WebDavProvider) {
				prov = (WebDavProvider) sp;
			} else {
				new UnexpectedValueException("StorageProvider","sp","is not a WebDAV provider").printDescription();
				return;
			}
			//		WebDavProvider prov = new WebDavProvider(providerName);
			( (WebDavProvider) prov).setWebDavBasePath(params[2]);
			try {
				//TestLog.v(this.getClass(), "Cursor.........");
				//LockFrameCalendar.getView().getPanel().setCursor(CursorManager.getWaitCursor());
				PswDialogView.getView().setCursor(CursorManager.getWaitCursor());
				SwingUtilities.getWindowAncestor(this).setCursor(CursorManager.getWaitCursor());
				getRootPane().getParent().setCursor(CursorManager.getWaitCursor());
				lockFrame.setCursor(CursorManager.getWaitCursor());

				if (pswString == null) {
					// get new password from properties
					pswString = CloudPropertyHandler.getPlainPasswordFromProvider(providerName, 
						Converter.bytes2hex(KeyDerivation.getSalt()), 
						Converter.bytes2hex(PeaProperties.getAlgoParamString()));
				}
				if (pswString != null) {
					String error = prov.authorize(params[0], pswString.toCharArray(), null);
					String[] fileNamesToDownload = null;
					if (error == null) {
						fileNamesToDownload = prov.getCloudFiles(PeaControl.getDialog().getSupportedExtensions());
						//( (WebDavProvider) prov).setWebDavBasePath(params[2]);
						CloudFileManager cfm = new CloudFileManager(rootWindow, prov, fileNamesToDownload);
						cfm.setVisible(true);
						lockFrame.setCursor(CursorManager.getDefaultCursor());
						String[] selectedFileNames = cfm.getSelectedFileNames();
						if (selectedFileNames == null ) {
							return;
						}

						if ( ! PeaProperties.getFileType().equals("file")) {
							//check if files already opened:
							// This becomes a conflict in Editor, Image and File, not in Calendar because of merging
							HashMap<String, HashSet<String>> map = ((FileTypePanel)JREProperties.getTypePanel()).getCloudFilePanel().getCloudControl().getCloudFileModel().getUsedCloudFileMap();
							if (map.containsKey(providerName)) {
								HashSet<String> set = map.get(providerName);
								StringBuilder builder = new StringBuilder();
								ArrayList<String> filesToAdd = new ArrayList<String>(); // add
								for (String s : selectedFileNames) {
									//s = CloudControl.getTmpFileNameForCloudFile(s, provider, null);
									if (set.contains(s)) {
										builder.append(s + "\n");
									} else {
										filesToAdd.add(s);
										// add to model
										((FileTypePanel)JREProperties.getTypePanel()).getCloudFilePanel().getCloudControl().getCloudFileModel().addUsedFileToProvider(providerName, new File(s).getName());
									}								
								}
								if (builder.length() > 0) {
									PeaDialog.showMessage(rootWindow, 
											providerName + ":\n"
													+ PeaProperties.getVmBridge().translate("file_already_open")
													+ "\n   " + new String(builder), 
													PeaProperties.getVmBridge().translate("error"), 0);
									TestLog.v(this.getClass(), "Already open: " + new String(builder));

									if (filesToAdd.size() == 0) {
										return;
									} else {
										selectedFileNames = Converter.arrayListToArray(filesToAdd);
									}
								}
							}
						} else {// File Lock PEA
							// check duplicates and ask to overwrite
							String[] fileNamesToCreate = new String[selectedFileNames.length];
							if (directoryName != null) {
								for (int i = 0; i < selectedFileNames.length; i++) {
									fileNamesToCreate[i] = directoryName + File.separator + selectedFileNames[i];
								}
								fileNamesToCreate = FileTools.askToOverwriteFiles(fileNamesToCreate);
								if (fileNamesToCreate == null) {
									return;
								}
								// Remove the directory
								int prePart = (directoryName + File.separator).length();
								for (String fn : fileNamesToCreate) {
									fn = fn.substring(prePart);
								}
								selectedFileNames = fileNamesToCreate;
							} else {
								new UnexpectedValueException("String", "directoryName", "is null").printDescription();
								PeaDialog.showMessage(this, languagesBundle.getString("unexpected_error") + ": (missing directory name)"
										+ "\n" + error, languagesBundle.getString("error"), 0);
								return;
							}
						}

						DownloadWorker dw = new DownloadWorker(lockFrame, providerName, 
								selectedFileNames, directoryName);
						dw.execute();

					} else {
						PeaDialog.showMessage(this, languagesBundle.getString("authorization_failed")
								+ "\n" + error, languagesBundle.getString("error"), 0);
					}
				} else {
					TestLog.e(getClass(), "unexpected error: missing password for " + providerName);
					TestLog.ve(getClass(), "Missing password for " + providerName, 5);
				}
			} finally {
				lockFrame.setCursor(CursorManager.getDefaultCursor());
			}

		} else if (command.startsWith("save_")) { // UPLOAD
			String providerName = command.substring("save_".length(), command.length());

			// get current used file names:
			String[] currentFileNamesWithPath = null;
			if (PeaControl.getDialog().getLockFrame() instanceof PeaLockFrame) {
				PeaLockFrame lf = (PeaLockFrame)PeaControl.getDialog().getLockFrame();
				currentFileNamesWithPath = lf.getCurrentFileNames();//lockFrame.getCurrentFileNames();
				//for(String s : currentFileNamesWithPath) {TestLog.v(this.getClass(), "file name to save: " + s);TestLog.v(this.getClass(), "Len: " + new File(s).length());}
			}  else {
				new UnexpectedValueException("frame", "LockFrame","is not instance of PeaLockFrame").printDescription();

			}	

			// check if password is present: Set if not
			//if (CloudPropertyHandler.getSingleLoginProperty(providerName, "password") == null) {
			if( CloudPropertyHandler.getPlainPasswordFromProvider(providerName, 
					Converter.bytes2hex(KeyDerivation.getSalt()), 
					Converter.bytes2hex(PeaProperties.getAlgoParamString())) == null) {
				// show password dialog
				char[] psw = PeaDialog.showPasswordDialog(rootWindow, PeaProperties.getVmBridge().translate("enter_cloud_password"), providerName, false);
				if (psw != null) {
					String error = CloudPropertyHandler.storeEncryptedPasswordFromProvider(
							CloudPropertyHandler.encryptPassword(new String(psw)), providerName, 
							Converter.bytes2hex(KeyDerivation.getSalt()), 
							Converter.bytes2hex(PeaProperties.getAlgoParamString()), null);
					if (error != null) {
						TestLog.e(getClass(), "unexpected error: could not set password for " + providerName);
						TestLog.ve(getClass(), "Error occurred for " + providerName, 5);
						return;
					}
				} else {
					TestLog.e(getClass(), "unexpected error: missing password for " + providerName);
					TestLog.ve(getClass(), "Missing password for " + providerName, 5);
					return;
				}
			}

			if (PeaProperties.getFileType().equals("text file")
					|| PeaProperties.getFileType().equals("image")) {
				// single file: not need to display selectable files
			} else { // Calendar Lock PEA or File Lock PEA
				// Ask user to choose files from current used files, get selected files:
				CurrentFileDisplay cfd = new CurrentFileDisplay(rootWindow, currentFileNamesWithPath);		
				// display failed files to upload
				currentFileNamesWithPath = cfd.getSelectedFileNames();
				if (cfd.getFailedFiles() != null && cfd.getFailedFiles().length > 0) {
					String message = PeaProperties.getVmBridge().translate("open_file") 
							+ " " + PeaProperties.getVmBridge().translate("failed") + "\n\n";
					for (String fileAndReason : cfd.getFailedFiles()) {
						message += fileAndReason + "\n";
					}
					PeaDialog.showMessage(rootWindow, message + "\n(failed files)", 
							PeaProperties.getVmBridge().translate("error"), 0);
				}
				if (currentFileNamesWithPath == null) {
					PeaDialog.showMessage(rootWindow, PeaProperties.getVmBridge().translate("no_valid_file_found"), 
							PeaProperties.getVmBridge().translate("error"), 0);
					return;
				} else {
					// check number and warn if required:
					long[] numberAndSize = FileTools.getNumberAndSize(currentFileNamesWithPath);
					if (numberAndSize[0] > 100 && numberAndSize[0] <= 300) {// File number
						boolean goOn = ExecutionTimeObserver.showWarningAndBreakOption(rootWindow, 
								PeaProperties.getVmBridge().translate("number_of_files") + " " + numberAndSize[0] + "\n" 
										+ PeaProperties.getVmBridge().translate("execution_time_warning"),
										false);
						if (goOn == false) {
							TestLog.e(CloudMenu.class, "Upload process cancelled...");
							return;
						}
					} else if (numberAndSize[0] > 300) {// File number)
						boolean goOn = ExecutionTimeObserver.showWarningAndBreakOption(rootWindow, 
								PeaProperties.getVmBridge().translate("number_of_files") + " " + numberAndSize[0] + "\n" 
										+ PeaProperties.getVmBridge().translate("extreme_time_warning"),
										true);
						if (goOn == false) {
							TestLog.e(CloudMenu.class, "Upload process cancelled...");
							return;
						}
					}
					// check size and warn if required:
					if (numberAndSize[1] > 1024 * 1024 * 2 && numberAndSize[1] <= 1024 * 1024 * 16) { // 1 MB
						boolean goOn = ExecutionTimeObserver.showWarningAndBreakOption(rootWindow, 
								PeaProperties.getVmBridge().translate("overall_size") + " " + numberAndSize[1] + "\n" 
										+ PeaProperties.getVmBridge().translate("execution_time_warning"),
										false);
						if (goOn == false) {
							TestLog.e(CloudMenu.class, "Upload process cancelled...");
							return;
						}
					} else if (numberAndSize[1] > 1024 * 1024 * 16) {// File size
						boolean goOn = ExecutionTimeObserver.showWarningAndBreakOption(rootWindow, 
								PeaProperties.getVmBridge().translate("overall_size") + " " + numberAndSize[1] + "\n" 
										+ PeaProperties.getVmBridge().translate("extreme_time_warning"),
										true);
						if (goOn == false) {
							TestLog.e(CloudMenu.class, "Upload process cancelled...");
							return;
						}
					}
				}
			}
			
			// create the file array to upload: 
			// copy files to tmp folder to upload
			// (File Lock PEA: copy to encrypt only the copy of the files,
			// notmthe files in FileTypePanel)
			int fileLen = currentFileNamesWithPath.length;
			ArrayList<File> filesToUploadList = new ArrayList<File>();

			String copyError = "";

			if (PeaProperties.getFileType().equals("file")) {

				// create the target file names
				String[] targetFileNames = new String[currentFileNamesWithPath.length];
				for (int i = 0; i < currentFileNamesWithPath.length; i++) {
					try {
						targetFileNames[i] = CloudControl.getTmpFileNameForCloudFile(
								new File(currentFileNamesWithPath[i]).getCanonicalFile().getName(), 
								providerName, currentFileNamesWithPath[i]);
					} catch (IOException e) {
						TestLog.e(getClass(), new ExceptionHelper(e).getInfos());
						// null files are handled in cipher
					}
				}

				// copy-encrypt the files:
				try {
					// set the wait cursor: 
					lockFrame.setCursor(CursorManager.getWaitCursor());
					FileComposer fc = null;
					if (JREProperties.getTypePanel() != null && JREProperties.getTypePanel() instanceof FileTypePanel) {
						fc = ((FileTypePanel)JREProperties.getTypePanel()).getFileComposer();
					} else {
						new UnexpectedValueException("Object", "typePanel", "is not FileTypePanel").printDescription();
					}
					String[] errorMessages = CipherStuff.getCipherMode().copyEncryptFiles(
							currentFileNamesWithPath, targetFileNames, 
							CipherStuff.getSessionKeyCrypt().getKey(), true, 
							fc);
					// collect the messages and failed files (index of errorMessages)
					for (int i = 0; i < errorMessages.length; i++) {
						if (errorMessages[i] != null && errorMessages.length > 0) {
							copyError +=  errorMessages[i] + " - " + currentFileNamesWithPath[i] + "\n";
						} else {
							filesToUploadList.add(new File(targetFileNames[i]));
						}
					}	

				} finally {
					lockFrame.setCursor(CursorManager.getDefaultCursor());
				}
				TestLog.v(this.getClass(), "file(s) encrypted");

			} else { // all other PEAs except File Lock PEA

				for (int i = 0; i < fileLen; i++) {
					File origFile = new File(currentFileNamesWithPath[i]);
					// Create a copy in tmp directory
					File destFile = null;
					try {
						destFile = new File(CloudControl.getTmpFileNameForCloudFile(
								origFile.getCanonicalFile().getName(), providerName, origFile.getAbsolutePath()));
					} catch (Exception e) {
						TestLog.e(getClass(), new ExceptionHelper(e).getInfos());
						copyError += origFile.getAbsolutePath() + ": "  + e.getLocalizedMessage() + "\n";
					}
					if (destFile != null) {
						if (origFile.getAbsolutePath().equals(destFile.getAbsolutePath())) {
							// do not copy the file itself
							TestLog.e(CloudMenu.class, "Break copy file to itself: " + origFile.getAbsolutePath());
							continue;
						}
						String e = FileTools.copyFiles(origFile, destFile);
						if (e != null) {
							copyError += e + "\n";
						} else {
							filesToUploadList.add(destFile);	
						}
					}
				}

			}
			// display failed files:
			if (copyError.length() > 0) {
				PeaDialog.showMessage(rootWindow, copyError + ("copy)"), 
						PeaProperties.getVmBridge().translate("error"), 0);
			}

			// upload the files
			if (filesToUploadList != null && filesToUploadList.size() > 0) {
				
				// upload the files
				File[] fileArray = filesToUploadList.toArray(new File[0]);

				new UploadWorker(JREProperties.getMainWindow(), 
						providerName, 
						null, 
						fileArray, 
						null).execute();				

			} else {
				TestLog.v(getClass(), "No file to upload...");
				//PeaDialog.showMessage(this, PeaProperties.getVmBridge().translate("no_valid_file_found"), 
				//		PeaProperties.getVmBridge().translate("error"), 0);
			}

		} else {
			TestLog.v(this.getClass(), "Invalid command in cloud menu");
		}
	}
	
	public void setRooTWindow(Window win) {
		this.rootWindow = win;
	}

	
    public AccessibleContext getAccessibleContext() {
        if (accessibleContext == null) {
            accessibleContext = new AccessibleCloudMenu();
        }
        return accessibleContext;
    }

    private class AccessibleCloudMenu extends AccessibleJMenu {
        //Inherit everything, override nothing.
    }
}
