package cologne.eck.all_peas.gui;

/*
 * Peafactory - Production of Password Encryption Archives
 * Copyright (C) 2020  Axel von dem Bruch
 * 
 * This library is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU General Public License as published 
 * by the Free Software Foundation; either version 2 of the License, 
 * or (at your option) any later version.
 * This library is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * See:  http://www.gnu.org/licenses/gpl-2.0.html
 * You should have received a copy of the GNU General Public License 
 * along with this library.
 */

import java.awt.Dimension;
import java.awt.Window;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Locale;

import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JEditorPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;


import cologne.eck.all_peas.data.PeaProperties;
import cologne.eck.tools.ExceptionHelper;
import cologne.eck.tools.TestLog;
import cologne.eck.tools.UnexpectedValueException;

@SuppressWarnings("serial")
public class HtmlFileViewer extends JDialog implements ActionListener {
	
	/**
	 * Shows a HTML file in a window. Looks for a file in the
	 * default language. If not found, shows tips in English.
	 * 
	 * @param fileName	the file name without extension html
	 * 					and without language
	 * @param loc		the location to show the dialog
	 */
	public HtmlFileViewer(String fileName, Window owner) {//Point loc) {

		super(owner);
		this.setModalityType(JDialog.ModalityType.APPLICATION_MODAL);

		JPanel contentPane = (JPanel) this.getContentPane();
		contentPane.setLayout(new BoxLayout(contentPane, BoxLayout.PAGE_AXIS));
		
		JEditorPane editorpane= new JEditorPane();
		editorpane.setFocusable(false);
		editorpane.setContentType("text/html");
		//editorpane.getDocument().putProperty("IgnoreCharsetDirective", Boolean.TRUE);
	    JScrollPane editorScrollPane = new JScrollPane(editorpane);
	    editorScrollPane.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
	    
	    String lang = Locale.getDefault().getLanguage();
	    
		String fileNameLang = fileName + "_" + lang + ".html";
		String fileNameDefault = fileName + ".html";// English version
		if (lang.equals("en")) {
			fileNameLang = fileNameDefault;
		}
	    
	    try {
	    	URL resUrl = HtmlFileViewer.class.getClassLoader().getResource("resources/" + fileNameLang);
			editorpane.setPage(resUrl);
		} catch (MalformedURLException e) {
			TestLog.e(getClass(), "Can't display guide for first use: " + fileNameLang);
			TestLog.e(getClass(), new ExceptionHelper(e).getInfos());
			return;
		} catch (IOException e) {
			// test default language (English) in jar file
			URL resUrl = HtmlFileViewer.class.getClassLoader().getResource("resources/" + fileNameDefault);
			try {
				editorpane.setPage(resUrl);
			} catch (IOException e2) {
				// test language in folder
				File file = new File("resources" + File.separator + fileNameLang);
				try {
					editorpane.setPage(file.toURI().toURL());
				} catch (MalformedURLException e1) {
					TestLog.e(getClass(), new ExceptionHelper(e1).getInfos());
					TestLog.e(getClass(), "Can't display guide for first use: " + fileNameLang);
					return;
				} catch (IOException e1) {
					// default language in folder
					file = new File("resources" + File.separator + fileNameDefault);
					if (file.exists()) {
						try {
							editorpane.setPage(file.toURI().toURL());
						} catch (IOException e3) {
							TestLog.e(getClass(), new ExceptionHelper(e3).getInfos());
							TestLog.e(getClass(), "Can't display guide for first use..." + fileNameDefault);
							return;
						}
					} else {
						TestLog.e(getClass(), "Can't find file to display guide for first use...");
						new UnexpectedValueException("File", "file", "does not exist: "+ fileNameDefault).printDescription();
						return;
					}
				}
			}
		}
	    editorpane.setEditable(false);
	    
	    contentPane.add(editorScrollPane);
	    
	    JButton okButton = new JButton(PeaProperties.getVmBridge().translate("ok"));
	    okButton.addActionListener(this);
	    okButton.setActionCommand("hideDialog");
	    okButton.setMaximumSize(new Dimension(200, 20));
	    contentPane.add(okButton);
	    
	    this.setLocation(owner.getLocationOnScreen());
	    this.setSize(new Dimension(500,450));

	    okButton.requestFocus();
	    okButton.requestFocusInWindow();

	    this.setVisible(true);	
	}

	@Override
	public void actionPerformed(ActionEvent ae) {

		if (ae.getActionCommand().equals("hideDialog")) {

			this.setVisible(false);
			this.dispose();

		}
		
	}

/*	public static void main(String[] args) {
			@SuppressWarnings("unused")
			HtmlFileViewer hfv = new HtmlFileViewer("first_time_guide_note_en.html", 
					new Point(300, 100));
	}*/

}
