package cologne.eck.all_peas.vm_specific;

/*
 * Peafactory - Production of Password Encryption Archives
 * Copyright (C) 2020  Axel von dem Bruch
 * 
 * This library is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU General Public License as published 
 * by the Free Software Foundation; either version 2 of the License, 
 * or (at your option) any later version.
 * This library is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * See:  http://www.gnu.org/licenses/gpl-2.0.html
 * You should have received a copy of the GNU General Public License 
 * along with this library.
 */

import java.net.InetAddress;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLClassLoader;
import java.net.UnknownHostException;
import java.time.LocalDateTime;
import java.time.ZoneId;

import cologne.eck.all_peas.files.FileComposer;
import cologne.eck.all_peas.files.FileTypePanel;
import cologne.eck.all_peas.gui.PeaDialog;
import cologne.eck.all_peas.gui.PswDialogView;
import cologne.eck.peafactory.crypto.HashStuff;
import cologne.eck.tools.Converter;
import cologne.eck.tools.EntropyPool;
import cologne.eck.tools.TestLog;
import cologne.eck.tools.Zeroizer;
import cologne.eck.all_peas.bridge.VMBridge;
import cologne.eck.all_peas.control.PeaControl;
import cologne.eck.all_peas.data.PeaProperties;

import java.awt.Component;
import java.awt.Window;
import java.io.File;
import java.util.ArrayList;
import java.util.Date;
import java.util.Locale;
import java.util.MissingResourceException;
import java.util.Random;
import java.util.ResourceBundle;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JPanel;

/**
 * @author axl
 *
 */
public class JREBridge implements VMBridge {
	
	
	private static ResourceBundle languagesBundle;	
	private static String language; 	
	private static String bundleFileName = "PeaLanguagesBundle";
	
/*	// pool for unique bytes (digest size)
	private static byte[] uniquePool = null;
	// counter where to get values from uniquePool
	private static int uniquePoolCounter = 0;
	// counter to produce new values for uniquePool
	private static int uniqueCounter = 0;*/
	
	private int entropyPoolCounter = 0;
	
	public JREBridge() {
		
	}

	@Override
	public String translate(String key) {
		if (JREProperties.getBundle() instanceof ResourceBundle) {
			
		}
		if (( (ResourceBundle)JREProperties.getBundle()).containsKey(key)){
			return ((ResourceBundle)JREProperties.getBundle()).getString(key);		
		} else {
			return key;
		}
	}
	
	@Override
	public void showUrgentMessage(Object owner, String message) {
		if (message == null) {			
			TestLog.e(getClass(), "Empty urgent error message");
			TestLog.ve(getClass(), "Empty urgent error message", 5);
			return;
		}
		if ( ! (owner instanceof Window)) {
			TestLog.ve(getClass(), "Window owner of urgent error message is not a Window", 5);
		}
		PeaDialog.showMessage(owner, message);
	}

	@Override
	public Object getProgressWidget(Object progressObject) {
		if (progressObject != null) {
			if (progressObject instanceof FileComposer) {
				return ((FileComposer) progressObject).getFileTypePanel();
			} else {
				return null;
			}
		} else {
			return null;
		}
	}

	@Override
	public long getAllSizeForProgress(Object progressObject) {
		if (progressObject instanceof FileComposer) {
			return ((FileComposer) progressObject).getFileModel().getAllSize();
		} else {
			return 0;
		}
	}

	@Override
	public void startProgressTask(Object ftp) {
		if (ftp != null) {
			if (ftp instanceof FileTypePanel) {
				((FileTypePanel) ftp).startProgressTask();
			} else {
			}
		}
	}

	@Override
	public void setProgress(Object progressWidget, double progress) {
		if (progressWidget != null) {
			if (progressWidget instanceof FileTypePanel) {
				((FileTypePanel) progressWidget).setProgressValue((int)progress);
			} else {
			}
		}	
	}

	@Override
	public void closeProgressTask(Object progressWidget) {
		if (progressWidget != null) {
			if (progressWidget instanceof FileTypePanel) {
				((FileTypePanel) progressWidget).closeProgressTask();
			} else {
			}
		}	
	}

	@Override
	public byte[] generatePersonalRandomBytes() {
		byte[] personalBytes = new byte[PERSONAL_BYTES_SIZE];
		int index = 0;

		// ip address of local host: 4 byte
		byte[] ipBytes = new byte[4];
		try {
			InetAddress ia = InetAddress.getLocalHost();
			ipBytes = ia.getAddress();
			if (ipBytes != null) {
				System.arraycopy(ipBytes,  0,  personalBytes, index,  ipBytes.length);
				index += ipBytes.length;
			} else {
				TestLog.e(getClass(), "RandomStuff: personal bytes for random seed: ip adress null");
			}
		} catch (UnknownHostException e) {
			TestLog.e(getClass(), "RandomStuff: " + e.toString() + ": personal bytes for random seed: ip adress failed");
		} catch (NullPointerException e) {
			TestLog.e(getClass(), "RandomStuff: " + e.toString() + ": personal bytes for random seed: ip adress failed");
		}
		
		// fill rest with system properties
		byte[] systemProps = ( System.getProperty("user.name") + System.getProperty("user.dir")
				+ System.getProperty("os.version")  ).getBytes();//+ System.getProperty("sun.boot.class.path")	
		for (int i = 0; i < systemProps.length && index < PERSONAL_BYTES_SIZE; i++) {
			personalBytes[index++] = systemProps[i];
		}			
		return personalBytes;
	}

	@Override
	public byte[] getAndUpdateEntopyPoolTableValue(int number) {
		EntropyPool rc = EntropyPool.getInstance();
		/*TestLog.e(getClass(), "started: " + rc.isStarted());
		if (rc.isStarted() == false) {
			
			if (PeaProperties.getWorkingMode().equals("-t")) {
				SecureRandom random = new SecureRandom();
				byte[] bytes = new byte[number];
				random.nextBytes(bytes);
				return bytes;
			} else {
				new UnexpectedValueException("EntropyPool", "rc", "is null").printDescription();
			}
		}*/

		int POOL_SIZE = EntropyPool.getPoolSize();
		
		byte[] result = new byte[number * 8];
		byte[] tableSeed = null;
		for (int i = 0; i < number; i++) {

			// get 8 bytes from one value of the table
			tableSeed = Converter.long2bytesBE(rc.getValue(true));
			entropyPoolCounter++;
			//System.out.print(" " + entropyPoolCounter);
			if (entropyPoolCounter >= POOL_SIZE){
				reseedEntropyPool(rc);
			}
			// store in result:
			System.arraycopy(tableSeed,  0, result, i * 8, tableSeed.length);
		}
		return result;
	}
	


	@Override
	public void reseedEntropyPool(Object ep) {
		//TestLog.o(getClass(), "\nreseed, counter: " + entropyPoolCounter);
		
		byte[] updateBytes = new byte[EntropyPool.getPoolSize() * 8];
		int hashLen = HashStuff.getHashAlgo().getDigestSize();
		// get old value from pool to hash
		byte[] reseedBytes = Converter.longs2bytesBE(((EntropyPool)ep).getValuesToReseed());// 32 bytes			
		// hash values
		byte[] hashedBytes = HashStuff.hash(reseedBytes);
		if (hashLen >= updateBytes.length) {
			System.arraycopy(hashedBytes,  0,  updateBytes,  0,  updateBytes.length);
		} else {
			
			int index = 0;
			while (index < updateBytes.length) {		
				hashedBytes = HashStuff.hash(hashedBytes);
				System.arraycopy(hashedBytes,  0,  updateBytes,  index,  
						(index + hashLen > updateBytes.length) ? (updateBytes.length - index) : hashedBytes.length);
				index += hashLen;
			}
		}

		// convert to long[]
		long[] reseedValues = Converter.bytes2longsLE(updateBytes);
		// reseed the pool
		((EntropyPool)ep).reseedPool(reseedValues);
		Zeroizer.zero(updateBytes);
		Zeroizer.zero(reseedBytes);
		Zeroizer.zero(hashedBytes);
		Zeroizer.zero(reseedValues);
		entropyPoolCounter = 0;
	}
	

	@Override
	public final void setLanguagesBundle(String _language) throws MalformedURLException {
		
		Locale defaultLocale = Locale.getDefault();
		Locale currentLocale;
		
		// i18n: 
		if (_language == null) {
			if (defaultLocale == null) { // openSuse
				defaultLocale = new Locale( System.getProperty("user.language") );
			}		
			language = defaultLocale.getLanguage(); // get default language
			if (language == null) { // fall back
				language = "en";
			}
		} else {
			language = _language;
		}
	    currentLocale = new Locale(language); // set default language as language
	    
	    // If PeaControl is not used inside of a pea 
	    // FilePanel uses this bundle
	    try {
	    	
	    	languagesBundle = ResourceBundle.getBundle("resources." + bundleFileName, currentLocale);
	    	
	    } catch (MissingResourceException mre) {

	    	// If called in PeaFactory, not in PEA
	    	try {	    
	    		//TestLog.o(getClass(), "Try to use bundle in folder resources (PeaFactory-Mode)");
	    	    File file = new File("resources");  
	    	    URL[] urls = {file.toURI().toURL()};  
	    	    ClassLoader loader = new URLClassLoader(urls);  
	    	    languagesBundle = ResourceBundle.getBundle(bundleFileName, currentLocale, loader);
	    	    if (languagesBundle == null) { // should not happen...
	    	    	languagesBundle = ResourceBundle.getBundle(bundleFileName, new Locale("en"), loader);
	    	    }
	    	    
	    	} catch  (MissingResourceException mre1) {

	    		TestLog.e(getClass(), "Resource Bundle PeaLanguagesBundle "+ currentLocale + " not found");
	    		
	    		try{
	    			// for PeaFactory: en
	    			File file = new File("resources");  
		    	    URL[] urls = {file.toURI().toURL()};  
		    	    ClassLoader loader = new URLClassLoader(urls);  
		    	    currentLocale = new Locale("en"); 
		    	    language = "en";
		    	    languagesBundle = ResourceBundle.getBundle(bundleFileName, currentLocale, loader);
	    		} catch (MissingResourceException mre2) {
	    			// for PEA: en
	    	    	languagesBundle = ResourceBundle.getBundle("resources." + bundleFileName, new Locale("en"));
	    		}
	    	}
	    } finally {
	    	JREProperties.setBundle(languagesBundle);
	    }
	}

	@Override
	public String[] askToOverwriteFiles(String[] fileNames) {
		if (fileNames == null) {
			return null;
		}
		ArrayList<String> resultList = new ArrayList<String>();
		
		JPanel panel = new JPanel();
		panel.setLayout(new BoxLayout(panel, BoxLayout.PAGE_AXIS));
		panel.add(new JLabel(PeaProperties.getVmBridge().translate("overwrite_existing_file") +":"));

		boolean existingFileFound = false;
		for (String fileName : fileNames) {
			if (fileName == null || fileName.length() == 0) {
				continue;
			}
			if (new File(fileName).exists() == true) {
				JCheckBox c = new JCheckBox(fileName);
				c.setSelected(true);
				panel.add(c);
				existingFileFound = true;
			} else {
				resultList.add(fileName);
			}
		}
		if (existingFileFound == false) {
			return fileNames;
		}
		Component comp = null;
		if (PswDialogView.getView().isVisible() == true
				&& PswDialogView.getView().isDisplayable()) {
			comp = PswDialogView.getView();
		} else {
			comp = (Component) PeaControl.getDialog().getLockFrame();
		}
		int result = PeaDialog.showQuestion(comp, panel, null, 0);
		if (result == 0) {
			Component[] comps = panel.getComponents();
			for (Component c : comps) {
				if (c instanceof JCheckBox) {
					JCheckBox check = (JCheckBox) c;
					if (check.isSelected() == true) {
						TestLog.o(getClass(), "Overwrite " + check.getText());
						resultList.add(check.getText());
					} else {
						TestLog.o(getClass(), "Do not overwrite " + check.getText());
					}
				}
			}
		} else if (result == 1) { // NO
			// return only files that do not exist
		} else {
			TestLog.e(getClass(), "Choose files to overwrite - Process cancelled...");
			return null;
		}
		if (resultList.isEmpty()) {
			return null;
		}
		return resultList.toArray(new String[0]); 
	}

	@Override
	public boolean askToOverwriteFile(File file) {
		if (file.exists() == true) {
			JPanel panel = new JPanel();
			panel.setLayout(new BoxLayout(panel, BoxLayout.PAGE_AXIS));
			panel.add(new JLabel(PeaProperties.getVmBridge().translate("overwrite_existing_file") +":"));
			panel.add(new JLabel(file.getAbsolutePath()));
			panel.add(Box.createVerticalStrut(5));

			Component comp = null;
			if (PswDialogView.getView().isVisible() == true
					&& PswDialogView.getView().isDisplayable()) {
				comp = PswDialogView.getView();
			} else {
				comp = (Component) PeaControl.getDialog().getLockFrame();
			}
			int result = PeaDialog.showQuestion(comp, panel, null, 0);
			if (result == 0) {
				return true;	
			} else {
				return false;
			}
		} else {
			return true;
		} 
	}

	@Override
	public LocalDateTime long2LocalDateTime(long timestamp) {
        return new Date(timestamp).toInstant()
                .atZone(ZoneId.systemDefault())
                .toLocalDateTime();
	}

	@Override
	public void createTestConditions() {
		EntropyPool.getInstance().updateThread(System.nanoTime());		
		EntropyPool.getInstance().stopCollection();		
		long[] reseedValues = new long[16];
		Random r = new Random();
		for (int i = 0; i < reseedValues.length;i++) {
			reseedValues[i] = r.nextLong();
		}
		EntropyPool.getInstance().reseedPool(reseedValues);
		EntropyPool.getInstance().stopCollection();
		
	}

	@Override
	public void exitWithError(Object ownerComponent, String message, boolean unencryptedFilesLeft) {
		TestLog.ve(this.getClass().getSimpleName(), "========= Program exists because of fatal error: \n"
				+ message + "probably unencrypted files left:" + unencryptedFilesLeft + " \n",
				8);
		TestLog.e(getClass(), "If you want to help to fix this bug, \nplease send this file to \npeafactory@eck.cologne");
		TestLog.o(getClass(), "Program exits with fatal error. See file error_log.txt");

		PeaDialog.showExitMessage(ownerComponent, message + "\n( -> " + PeaProperties.getVmBridge().translate("file") + ":  error_log.txt)", unencryptedFilesLeft);
		System.exit(1);
	}



}
