package cologne.eck.note_pea;

/*
 * PeaFactory - Production of Password Encryption Archives
 * Copyright (C) 2015  Axel von dem Bruch
 * 
 * This library is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU General Public License as published 
 * by the Free Software Foundation; either version 2 of the License, 
 * or (at your option) any later version.
 * This library is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * See:  http://www.gnu.org/licenses/gpl-2.0.html
 * You should have received a copy of the GNU General Public License 
 * along with this library.
 */

import java.awt.Component;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.accessibility.AccessibleContext;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.text.JTextComponent;

import cologne.eck.all_peas.data.PeaProperties;



class CopyCutPastePopup extends JPopupMenu implements ActionListener {
	
	private static CopyCutPastePopup popup;
	
	private static final long serialVersionUID = 1L;
	private JMenuItem cut;
	private JMenuItem copy;
	private JMenuItem paste;
	private JTextComponent textComponent;
		
	// to register if a CopyCutPastePopup is displayed or not
	//private static boolean popupStarted = false;
	
	private CopyCutPastePopup (Component invoker) {
		
		popup = this;
		int fontSize = PeaProperties.getFontSize();//PswDialogView.getFontsize();
		
		textComponent = (JTextComponent)invoker;
		cut = new JMenuItem(PeaProperties.getVmBridge().translate("cut"));
	    cut.addActionListener(this);
	    cut.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, fontSize));
	    this.add(cut);
	    this.addSeparator();
	    copy = new JMenuItem(PeaProperties.getVmBridge().translate("copy"));
	    copy.addActionListener(this);
	    copy.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, fontSize));
	    this.add(copy);
	    this.addSeparator();
	    paste = new JMenuItem(PeaProperties.getVmBridge().translate("paste"));
	    paste.addActionListener(this);
	    paste.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, fontSize));
	    this.add(paste);		    
	    JPopupMenu.setDefaultLightWeightPopupEnabled(true);
	    this.setBorderPainted(true);		    
	    this.setInvoker(invoker);		    
	    this.pack();
	}
	
	protected static CopyCutPastePopup getInstance(Component invoker) {
		if (popup == null) {
			popup = new CopyCutPastePopup(invoker);
		}
		return popup;
	}


	@Override
	public void actionPerformed(ActionEvent ape) {
		
		if (ape.getSource() == cut) {
			textComponent.cut();
			//setPopupStarted(false);
			popup.setVisible(false);
		}
		if (ape.getSource() == copy) {
			textComponent.copy();
			//setPopupStarted(false);
			popup.setVisible(false);
		}
		if (ape.getSource() == paste) {
			textComponent.paste();
			//setPopupStarted(false);
			popup.setVisible(false);
		}		
	}
	
    public AccessibleContext getAccessibleContext() {
        if (accessibleContext == null) {
            accessibleContext = new AccessibleCopyCutPastePopup();
        }
        return accessibleContext;
    }

    @SuppressWarnings("serial")
	private class AccessibleCopyCutPastePopup extends AccessibleJPopupMenu {
        //Inherit everything, override nothing.
    }
}
