package cologne.eck.note_pea;

/*
 * Peafactory - Production of Password Encryption Archives
 * Copyright (C) 2015  Axel von dem Bruch
 * 
 * This library is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU General Public License as published 
 * by the Free Software Foundation; either version 2 of the License, 
 * or (at your option) any later version.
 * This library is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * See:  http://www.gnu.org/licenses/gpl-2.0.html
 * You should have received a copy of the GNU General Public License 
 * along with this library.
 */

/**
 * Main frame of editor pea.  
 */

import java.awt.*;
import java.awt.event.*;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;

import javax.accessibility.AccessibleContext;
import javax.swing.*;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.text.*;
import javax.swing.text.rtf.RTFEditorKit;
import javax.swing.undo.*;

import cologne.eck.all_peas.cloud.CloudControl;
import cologne.eck.all_peas.cloud.StorageProvider;
import cologne.eck.all_peas.control.ContentUtilities;
import cologne.eck.all_peas.control.PathFileManager;
import cologne.eck.all_peas.control.PeaControl;
import cologne.eck.all_peas.data.PeaProperties;
import cologne.eck.all_peas.files.CloudFilePanel;
import cologne.eck.all_peas.gui.CursorManager;
import cologne.eck.all_peas.gui.PeaDialog;
import cologne.eck.all_peas.gui.PeaLockFrame;
import cologne.eck.all_peas.gui.PswDialogView;
import cologne.eck.all_peas.vm_specific.JREProperties;
import cologne.eck.peafactory.crypto.CipherStuff;
import cologne.eck.tools.Converter;
import cologne.eck.tools.ExceptionHelper;
import cologne.eck.tools.PropertyHandler;
import cologne.eck.tools.ReadResources;
import cologne.eck.tools.TestLog;
import cologne.eck.tools.UnexpectedValueException;


@SuppressWarnings("serial")
public final class LockFrameEditor extends PeaLockFrame 
			implements 
			WindowListener, // windowClosingCommands
			DocumentListener, // to register if changes in text are saved
			ChangeListener { // for JTabbedPane focus change
	
	// this frame
	private static LockFrameEditor frame;
	
	/*
	 * The label for messages and errors
	 */
	private static JLabel messageLabel;

	/**
	 * The tab pane containing one tab for each encrypted
	 * text file. Only one file at the same time is decrypted
	 * and displayed: the file content associated with the 
	 * selected and focused tab. 
	 */
	private static JTabbedPane tabPane;
	
	/** this is the actual focused tab with EditorKit, MouseListener, UndoManager
	 * before the focus is changing
	 */
	private static TabComponent lastFocusTab;

	/**
	 * UndoManager, which is always registered to the currently
	 * selected and focused tab (its JTextPane)
	 */
	protected static UndoManager manager;// = new UndoManager();
	protected static RTFEditorKit kit = null;
	
	// if the content of the current document was modified: true
	private boolean docChangeUnsaved = false;
	
	/**
	 * The associated password dialog, which calls the instance
	 * of this class
	 */
	protected static EditorControl dialog;


	private LockFrameEditor(EditorControl _dialog, Point location) {
				
		frame = this;
		dialog = _dialog;
		
	    this.setIconImage( PswDialogView.getImage() );
	    this.setTitle("NotebookPEA");
	    
	    frame.setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
	    
		this.addWindowListener(this);// for windowClosing
		
		JPanel contentPane = new JPanel();
		contentPane.setLayout(new BoxLayout(contentPane, BoxLayout.PAGE_AXIS));
		this.setContentPane(contentPane);
		
		JPanel menuPanel = new JPanel();
		menuPanel.setLayout(new BoxLayout(menuPanel, BoxLayout.X_AXIS));
		LockFrameEditorMenu menu = new LockFrameEditorMenu(this);		
		menuPanel.add(menu);
		menuPanel.setMinimumSize(new Dimension (300, 40));
		menuPanel.add(Box.createHorizontalGlue());
		contentPane.add(menuPanel);
		contentPane.add(Box.createVerticalStrut(10));
		
		JPanel buttonMenuPanel = new JPanel();
		buttonMenuPanel.setLayout(new BoxLayout(buttonMenuPanel, BoxLayout.X_AXIS));
		buttonMenuPanel.setMinimumSize(new Dimension(800, 30));	
		buttonMenuPanel.setMaximumSize(new Dimension(800, 30));		
				
		buttonMenuPanel.add(Box.createHorizontalGlue());		
			// does not work: JTextPane does not save alignment
		JButton leftButton = new JButton();
		leftButton.setAction(new StyledEditorKit.AlignmentAction("=  ", 0));
		leftButton.setMargin(new Insets(0,0,0,15));//top-left-bottom-right
//		leftButton.setBackground(Color.WHITE);
		leftButton.setAlignmentX(LEFT_ALIGNMENT);
		leftButton.setToolTipText(PeaProperties.getVmBridge().translate("left"));
		leftButton.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 12));
		//leftButton.setMaximumSize(new Dimension(25, 20));
		buttonMenuPanel.add(leftButton);
		
		JButton centerButton = new JButton();
		centerButton.setAction(new StyledEditorKit.AlignmentAction(" = ", 1));
		centerButton.setMargin(new Insets(0,7,0,7));//top-left-bottom-right
//		centerButton.setBackground(Color.WHITE);
		centerButton.setAlignmentX(LEFT_ALIGNMENT);
		centerButton.setToolTipText(PeaProperties.getVmBridge().translate("center"));
		centerButton.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 12));
		buttonMenuPanel.add(centerButton);		
		
		JButton rightButton = new JButton();
		rightButton.setAction(new StyledEditorKit.AlignmentAction("  =", 2));
		rightButton.setMargin(new Insets(0,15,0,0));//top-left-bottom-right
//		rightButton.setBackground(Color.WHITE);
		rightButton.setAlignmentX(LEFT_ALIGNMENT);
		rightButton.setToolTipText(PeaProperties.getVmBridge().translate("right"));
		rightButton.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 12));
		buttonMenuPanel.add(rightButton);
		
		buttonMenuPanel.add(Box.createHorizontalStrut(5));
		
		/*
		 * Problem: On Mac the background color disappears, 
		 * if not setOpaque(true) but on other systems the
		 * focus is not visible then... Setting the "text" 
		 * to "*" and the foreground is a workaround for this.
		 */
		
		JButton colorButton0 = new JButton("*");
		colorButton0.getAccessibleContext().
	    setAccessibleDescription(
	    	    "Clicking this component sets the font color for a selected text to gold.");
//colorButton0.setContentAreaFilled(true);
		colorButton0.setAction(new StyledEditorKit.ForegroundAction("",new Color(139,139, 0)));
		colorButton0.setBackground(new Color(139,139, 0));//yellow4
		colorButton0.setForeground(new Color(139,139, 0));
		colorButton0.setPreferredSize(new Dimension(20,20));
		colorButton0.setMaximumSize(new Dimension(20,20));
		//colorButton0.setOpaque(true);// more L&F-secure
		colorButton0.setBorderPainted(false);
//		colorButton0.setBorder(new LineBorder(new Color(139,139, 0), 8));// more L&F-secure
		buttonMenuPanel.add(colorButton0);

		JButton colorButton1 = new JButton("*");
		colorButton1.getAccessibleContext().
	    setAccessibleDescription(
	    	    "Clicking this component sets the font color for a selected text to orange.");
		colorButton1.setAction(new StyledEditorKit.ForegroundAction("",new Color(205,102,0)));//Color.ORANGE));
		colorButton1.setBackground(new Color(205,102,0));//darkorange2
		colorButton1.setForeground(new Color(205,102,0));
		colorButton1.setPreferredSize(new Dimension(20,20));
		colorButton1.setMaximumSize(new Dimension(20,20));
		//colorButton1.setOpaque(true);
		colorButton1.setBorderPainted(false);
	//	colorButton1.setBorder(new LineBorder(new Color(205,102,0), 8));
		colorButton1.setFocusable(true);
		buttonMenuPanel.add(colorButton1);
		
		JButton colorButton2 = new JButton("*");
		colorButton2.getAccessibleContext().
	    setAccessibleDescription(
	    	    "Clicking this component sets the font color for a selected text to red.");
		colorButton2.setAction(new StyledEditorKit.ForegroundAction("",new Color(205,0,0)));
		colorButton2.setBackground(new Color(205,0,0));//red3
		colorButton2.setForeground(new Color(205,0,0));
		colorButton2.setPreferredSize(new Dimension(20,20));
		colorButton2.setMaximumSize(new Dimension(20,20));
		//colorButton2.setOpaque(true);
		colorButton2.setBorderPainted(false);
		//colorButton2.setBorder(new LineBorder(new Color(205,0,0), 8));
		buttonMenuPanel.add(colorButton2);

		JButton colorButton3 = new JButton("*");
		colorButton3.getAccessibleContext().
	    setAccessibleDescription(
	    	    "Clicking this component sets the font color for a selected text to pink.");
		colorButton3.setAction(new StyledEditorKit.ForegroundAction("",new Color(205,0,205)));
		colorButton3.setBackground(new Color(205,0,205));//magenta3
		colorButton2.setForeground(new Color(205,0,205));//magenta3
		colorButton3.setPreferredSize(new Dimension(20,20));
		colorButton3.setMaximumSize(new Dimension(20,20));
		//colorButton3.setOpaque(true);
		colorButton3.setBorderPainted(false);
		//colorButton3.setBorder(new LineBorder(new Color(205,0,205), 8));
		buttonMenuPanel.add(colorButton3);
		
		JButton colorButton4 = new JButton("*");
		colorButton4.getAccessibleContext().
	    setAccessibleDescription(
	    	    "Clicking this component sets the font color for a selected text to blue.");
		colorButton4.setAction(new StyledEditorKit.ForegroundAction("",Color.BLUE));
		colorButton4.setBackground(Color.BLUE);
		colorButton4.setForeground(Color.BLUE);
		colorButton4.setPreferredSize(new Dimension(20,20));
		colorButton4.setMaximumSize(new Dimension(20,20));
		//colorButton4.setOpaque(true);
		colorButton4.setBorderPainted(false);
		//colorButton4.setBorder(new LineBorder(Color.BLUE, 8));
		buttonMenuPanel.add(colorButton4);
		
		JButton colorButton5 = new JButton("*");
		colorButton5.getAccessibleContext().
	    setAccessibleDescription(
	    	    "Clicking this component sets the font color for a selected text to bright blue.");
		colorButton5.setAction(new StyledEditorKit.ForegroundAction("",new Color(0,139,139)));
		colorButton5.setBackground(new Color(0,139,139));//cyan4
		colorButton5.setForeground(new Color(0,139,139));//cyan4
		colorButton5.setPreferredSize(new Dimension(20,20));
		colorButton5.setMaximumSize(new Dimension(20,20));
		//colorButton5.setOpaque(true);
		colorButton5.setBorderPainted(false);
		//colorButton5.setBorder(new LineBorder(new Color(0,139,139), 8));
		buttonMenuPanel.add(colorButton5);
		
		JButton colorButton6 = new JButton("*");
		colorButton6.getAccessibleContext().
	    setAccessibleDescription(
	    	    "Clicking this component sets the font color for a selected text to green.");
		colorButton6.setAction(new StyledEditorKit.ForegroundAction("",new Color(0,139,0)));
		colorButton6.setBackground(new Color(0,139,0));// green4
		colorButton6.setForeground(new Color(0,139,0));// green4
		colorButton6.setPreferredSize(new Dimension(20,20));
		colorButton6.setMaximumSize(new Dimension(20,20));
		//colorButton6.setOpaque(true);
		colorButton6.setBorderPainted(false);
		//colorButton6.setBorder(new LineBorder(new Color(0,139,0), 8));
		buttonMenuPanel.add(colorButton6);		
		
		JButton colorButton8 = new JButton("*");
		colorButton8.getAccessibleContext().
	    setAccessibleDescription(
	    	    "Clicking this component sets the font color for a selected text to gray.");
		colorButton8.setAction(new StyledEditorKit.ForegroundAction("",Color.DARK_GRAY));
		colorButton8.setBackground(Color.DARK_GRAY);
		colorButton8.setForeground(Color.DARK_GRAY);
		colorButton8.setPreferredSize(new Dimension(20,20));
		colorButton8.setMaximumSize(new Dimension(20,20));
		//colorButton8.setOpaque(true);
		colorButton8.setBorderPainted(false);
		//colorButton8.setBorder(new LineBorder(Color.DARK_GRAY, 8));
		buttonMenuPanel.add(colorButton8);		
		
		JButton colorButton9 = new JButton("*");
		colorButton9.getAccessibleContext().
	    setAccessibleDescription(
	    	    "Clicking this component sets the font color for a selected text to black.");
		colorButton9.setAction(new StyledEditorKit.ForegroundAction("",Color.BLACK));
		colorButton9.setBackground(Color.BLACK);
		colorButton9.setForeground(Color.BLACK);
		colorButton9.setPreferredSize(new Dimension(20,20));
		colorButton9.setMaximumSize(new Dimension(20,20));
		//colorButton9.setOpaque(true);
		colorButton9.setBorderPainted(false);
		//colorButton9.setBorder(new LineBorder(Color.BLACK, 8));
		buttonMenuPanel.add(colorButton9);
		
		buttonMenuPanel.add(Box.createHorizontalStrut(5));

		
		JButton nineButton = new JButton();
		nineButton.setAction(new StyledEditorKit.FontSizeAction("" + 9, 9));
		nineButton.setMargin(new Insets(4,3,4,3));//top-left-bottom-right
		nineButton.setAlignmentX(LEFT_ALIGNMENT);
		nineButton.setToolTipText(PeaProperties.getVmBridge().translate("font_size") + " " + 9);
		nineButton.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 9));
		buttonMenuPanel.add(nineButton);
		
		JButton twelveButton = new JButton();
		twelveButton.setAction(new StyledEditorKit.FontSizeAction("" + 12, 12));
		twelveButton.setMargin(new Insets(2,0,3,0));//top-left-bottom-right
		twelveButton.setAlignmentX(LEFT_ALIGNMENT);
		twelveButton.setToolTipText(PeaProperties.getVmBridge().translate("font_size") + " " + 12);
		twelveButton.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 12));
		buttonMenuPanel.add(twelveButton);		
		
		JButton fiveteenButton = new JButton();
		fiveteenButton.setAction(new StyledEditorKit.FontSizeAction("" + 15, 15));
		fiveteenButton.setMargin(new Insets(1,0,1,0));//top-left-bottom-right
		fiveteenButton.setAlignmentX(LEFT_ALIGNMENT);
		fiveteenButton.setToolTipText(PeaProperties.getVmBridge().translate("font_size") + " " + 14);
		fiveteenButton.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 15));
		buttonMenuPanel.add(fiveteenButton);
		
		buttonMenuPanel.add(Box.createHorizontalStrut(5));

		
		JButton boldButton = new JButton();
		boldButton.setAction(new StyledEditorKit.BoldAction());
		boldButton.setText(PeaProperties.getVmBridge().translate("bold_mnemonic"));
		boldButton.setToolTipText(PeaProperties.getVmBridge().translate("bold"));
		boldButton.setFont(new Font(Font.SANS_SERIF, Font.BOLD, 12));
		boldButton.setMargin(new Insets(2,5,2,5));//top-left-bottom-right
		buttonMenuPanel.add(boldButton);

		JButton italicButton = new JButton();
		italicButton.setAction(new StyledEditorKit.ItalicAction());
		italicButton.setText(PeaProperties.getVmBridge().translate("italic_mnemonic"));
		italicButton.setToolTipText(PeaProperties.getVmBridge().translate("italic"));
		italicButton.setFont(new Font(Font.SANS_SERIF, Font.ITALIC, 12));
		italicButton.setMargin(new Insets(2,5,2,5));//top-left-bottom-right
		buttonMenuPanel.add(italicButton);

		JButton underlineButton = new JButton();
		underlineButton.setAction(new StyledEditorKit.UnderlineAction());
		underlineButton.setText(PeaProperties.getVmBridge().translate("underline_mnemonic"));
		underlineButton.setToolTipText(PeaProperties.getVmBridge().translate("underline"));
		underlineButton.setFont(new Font(Font.SANS_SERIF, Font.PLAIN, 12));
		underlineButton.setMargin(new Insets(2,5,2,5));//top-left-bottom-right
		underlineButton.setMaximumSize(underlineButton.getPreferredSize());//new Dimension(30,30));
		buttonMenuPanel.add(underlineButton);

		contentPane.add(buttonMenuPanel);

		// tab are added in PsDialogEditor startDecryption
		tabPane = new JTabbedPane();
		contentPane.add(tabPane);
		
		JPanel messagePanel= new JPanel();
		messagePanel.setLayout(new BoxLayout(messagePanel, BoxLayout.LINE_AXIS));
		messageLabel = new JLabel();
		messagePanel.add(messageLabel);
		messagePanel.add(Box.createHorizontalGlue());
		contentPane.add(messagePanel);

		this.setLocation(location);
		
		// Notebook needs a minimum width to display tool bar, others than other PEAs
		pack();
		int[] dim  = JREProperties.getFrameDim();
		if (dim[0] > PswDialogView.getScreenwidth()) {
			dim[0] = (int) PswDialogView.getScreenwidth();
		}
		if (dim[1] > PswDialogView.getScreenheight()) {
			dim[1] = (int) PswDialogView.getScreenheight();
		}

		this.setSize(new Dimension(dim[0], dim[1]));
		menu.setRootWindowForCloudMenu(this);

		docChangeUnsaved = false;
/*		if (dim.width >= 600) {
			this.setMinimumSize(PeaProperties.getFrameDim());
		} else {*/
//			this.setMinimumSize(new Dimension(500, dim.height));
		//}
		//TestLog.v(this.getClass(), UIManager.getLookAndFeel().getName());
		// some L&Fs need more space to display the button verticalBar
/*		String lfs = UIManager.getLookAndFeel().getName();
		if (lfs.equals("Metal")){
			this.setMinimumSize(new Dimension(450, 500));
		} else if (lfs.equals("Nimbus")){
			this.setMinimumSize(new Dimension(600, 500));
		} else if (lfs.contains("Motif")){
			this.setMinimumSize(new Dimension(600, 500));
		} else if (lfs.startsWith("GTK")){
			this.setMinimumSize(new Dimension(500, 500));
		} else if (lfs.equals("Windows")){
			this.setMinimumSize(new Dimension(500, 500));
		} else {
			this.setMinimumSize(new Dimension(450, 500));
		}*/
	}
	/**
	 * Get the current instance of ContentUtilities or create a
	 * new one if no instance exists - there is always only
	 * one instance. 
	 * 
	 * @param pswDialog		the associated password dialog
	 * @param loc			the location on screen 
	 * 
	 * @return				the instance of ContentUtilities
	 */
	protected static LockFrameEditor getInstance(EditorControl pswDialog, Point loc) {
		LockFrameEditor frameInstance = null; 
		if (frame == null) {
			frameInstance = new LockFrameEditor(pswDialog, loc);
			LockFrameEditorMenu.setLockFrame(frameInstance);
		} else {
			frameInstance = frame;
		}
		return frameInstance;
	}


/*	// for testing
	public static void main(String[] args) {
		PswDialogView.setUI(false);
		// customize button margin: Done in CalendarControl
		UIManager.put("Button.contentMargins", new Insets( 5,9,5,9) );		

		LockFrameEditor lfe = LockFrameEditor.getInstance(null, new Point(100,100));//CalendarControl pswDialog, Point loc)
		lfe.setVisible(true);
		
	}*/

	/**
	 * Create a new tab for the given file name and add
	 * it to the JTabbedPane. Unless the tab gets the focus,
	 * the content of the file is not decrypted and displayed. 
	 * 
	 * @param fileName	the name of the text file
	 */
	private void createNewTab(HashSet<String> set){
		
		//if (new File(fileName).isFile()) {
			@SuppressWarnings("unused") // add tab in method, not in constructor...
			TabComponent tab = new TabComponent(tabPane, set);
		//}
	}
	
 
	/**
	 * pack the frame if new components were added or removed
	 */
	public static void packFrame(){
		frame.pack();
	}

	
	/**
	 * Manually add ChangeListener after all tab were added, 
	 * set focus to first tab
	 */
	public void setChangeListener(){

/*for (int i = 0; i < tabPane.getTabCount(); i++) {
	Component c = tabPane.getTabComponentAt(i);
	TestLog.v(this.getClass(), i + " " + c.getName());
}*/
		if (tabPane.getTabCount() > 0) {
			// check if there is a selected component
			if (tabPane.getSelectedComponent() != null) {
				// try to set focus on selected component
				if (tabPane.getSelectedComponent() instanceof TabComponent) {
					String focusError = changeFocus((TabComponent) tabPane.getSelectedComponent());
					if (focusError != null) { // set focus failed
						// try to set focus on component at index 0
						if (tabPane.getTabCount() > 0 && tabPane.getTabComponentAt(0) instanceof TabComponent) {
							tabPane.setSelectedIndex(0);
							focusError = changeFocus((TabComponent) tabPane.getSelectedComponent());
							if (focusError != null) { // set focus failed
								// try to find any valid tab to set focus
								int tabCount = tabPane.getTabCount();
								TabComponent tab = null;
								// check all tabs:
								for (int i = 0; i < tabCount; i++) {
									Component cTab = tabPane.getComponentAt(i);// the tab title panel
									Component cTabComp = tabPane.getTabComponentAt(i);// the TabComponent
									if (cTab != null && cTabComp != null) {
										if (cTabComp instanceof TabComponent) {
											// valid tab found:
											tab = (TabComponent) tabPane.getComponentAt(i);
											break;
										}
									}
								}
								if (tab == null) { // no tab found:
									new UnexpectedValueException("TabComponent","tab", "is not null - can not set focus on tab").printDescription();
									// close program, otherwise a text pane without function appears
									windowClosingCommands();
								}
								focusError = changeFocus(tab);
								if (focusError != null) { // no valid tab found
									new UnexpectedValueException("String","focusError", "is not null - can not set focus on tab").printDescription();
									windowClosingCommands();
								}
							}
						} else {
							//TestLog.ve(getClass(), "Can not set selected component. tabPane size: " + tabPane.getTabCount(), false);
							if (tabPane.getTabCount() > 0) {
								TestLog.e(getClass(), "Can not set select component: " + tabPane.getTabComponentAt(0).getClass().getSimpleName());
							}
						}
					}
				} else {
					TestLog.ve(getClass(), "not a TabComponent: " + tabPane.getSelectedComponent().getClass().getName(), 5);
				}
			} else {
				tabPane.setSelectedIndex(0);
				changeFocus((TabComponent) tabPane.getSelectedComponent());
			}
		}
		tabPane.addChangeListener(this);
		Component c =  tabPane.getSelectedComponent();//.getTabComponentAt(tabPane.getSelectedIndex());
		if (c instanceof TabComponent) {
			//lastFocusTab = (TabComponent) c;
			setLastFocusTab( (TabComponent) c);
		}
	}
	

	@Override
	public void markUnsavedContentChanges() {
		docChangeUnsaved = true;
	}
	@Override
	public void markContentIsSaved() {
		docChangeUnsaved = false;
	}
	/**
	 * Check if the text of the last selected tab
	 * was modified and not saved manually and 
	 * save the text in this case. 
	 * 
	 * @return	true if content is to be saved
	 */
	public boolean checkUnsavedContentChanges() {
		if (docChangeUnsaved == true) {
			int n = JOptionPane.showConfirmDialog(
				    this,
				    PeaProperties.getVmBridge().translate("save_modifications"),//"Save changes?",
				    PeaProperties.getVmBridge().translate("unsaved_modifications"),//"Unsaved Changes",
				    JOptionPane.YES_NO_OPTION);
			if (n == JOptionPane.YES_OPTION) {
//TODO TEST
				HashSet<String> tabSet = lastFocusTab.getFileNames();
				byte[] plainBytes = getPlainBytes();
				byte[] cipherBytes = CipherStuff.getInstance().encrypt(plainBytes, // is zeroized
						CipherStuff.getSessionKeyCrypt().getKey(), // keyMaterial is overwritten
						true, null ); // key is stored encrypted
				boolean cloudFileToSave = false;
				for (String nameInTab : tabSet) {					
					ContentUtilities.saveContent(null, null, cipherBytes, nameInTab, true);
					if (nameInTab.contains(CloudControl.getTmpDir())) {
						cloudFileToSave = true;
					}
				}
				// if cloud files: wait until all UploadWorkers started
				// and set running process....
				if (cloudFileToSave == true) {
					// wait a short time, because all UploadWorkers
					// must have initialized and set running processes
					try {
						Thread.sleep(50);
					} catch (InterruptedException e) {
						//e.printStackTrace();
					}
				}
				docChangeUnsaved = false;
				return true;
			} else {
				return false;
			}
		} else {
			return false;
		}
	}
	
	/**
	 * If a user sets the focus on a new tab manually
	 * or if the focus is set by the program:
	 * Change the title of the new selected tab and
	 * - set EditorKit
	 * - add MouseListener for CopyCutPastePopup
	 * - add DocumentListener to register unsaved changes
	 * - add Redo-Undo-Manager
	 * - register KeyStrokes
	 * and remove all this from the previously focused tab
	 * 
	 * @param newFocusTab		the tab that should get the focus
	 * 
	 * @return an error message if the setting of the focus failed
	 */
	protected String changeFocus(TabComponent newFocusTab) {

		if (newFocusTab == null) {
			docChangeUnsaved = false;
			TestLog.e(LockFrameEditor.class, "Missing tab to set focus");
			return "Missing tab to set focus";
		}
		
		if (tabPane.getTabCount() == 0 ){
			TestLog.e(LockFrameEditor.class, "tabPane is empty");
			docChangeUnsaved = false;
			return "Empty tab pane";
		}

		// Handle the previously selected tab: 
		if (	lastFocusTab != null 								// exists
				&& (tabPane.indexOfComponent(lastFocusTab) != -1)	// is in tabPane
		//		&& lastFocusTab.getFileNameWithoutPath() != null
				) {			// was not closed before
			
			// ask to save text if text was changed and not saved manually:
			checkUnsavedContentChanges();
			
			// set no image and gray background: 
			lastFocusTab.setTabTitle(false);
			
			// get the associated JTextPane:
			JTextPane oldTextPane = lastFocusTab.getTextPane();
			
			// remove DocumentListener:
			oldTextPane.getDocument().removeDocumentListener(this);
			
			// remove Cut-Copy-Paste.Menue: 
			oldTextPane.removeMouseListener(lastFocusTab);

			// remove redo and undo manager:
			oldTextPane.getDocument().removeUndoableEditListener(manager);
			
			oldTextPane.setText("");
			   
			//remove keyStroke: STRG + R, STRG + W
			oldTextPane.unregisterKeyboardAction(KeyStroke.getKeyStroke(
			            KeyEvent.VK_R, InputEvent.CTRL_DOWN_MASK ));
			oldTextPane.unregisterKeyboardAction(KeyStroke.getKeyStroke(
			            KeyEvent.VK_W, InputEvent.CTRL_DOWN_MASK ));	
		} else {
			//new UnexpectedValueException("TabComponent", "lastFocusTab", "is missing").printDescription();
		}

		// get the JTextPane of the new tab:
		JTextPane textPane = newFocusTab.getTextPane();		
		
		// set EditorKit:
		kit = new RTFEditorKit();
		textPane.setEditorKit(kit);
		
		// set the text
		String errorMessage = newFocusTab.setText();
		boolean newFocusTabFailed = false;
		if (errorMessage != null) { // decryption failed
			
			newFocusTabFailed = true;
			
			PeaDialog.showMessage(this, newFocusTab.getFileNameWithoutPath() + ": "
			+ PeaProperties.getVmBridge().translate("decryption_failed")
			+ "\n" + errorMessage, PeaProperties.getVmBridge().translate("error")
			, 0);
			
			TestLog.e(LockFrameEditor.class, "decryption failed for new focused tab: " + newFocusTab.getFileNameWithoutPath()
					+ ", " + errorMessage);
			docChangeUnsaved = false;

			// remove file name from list:
			ArrayList<String> list = PeaControl.getAvailableFileNames();
			HashSet<String> tabSet = newFocusTab.getFileNames();
			boolean removedFromSet = false;
			if (tabSet != null && tabSet.isEmpty() == false) {
				for (String nameInSet : tabSet) {
					if (list.contains(nameInSet)) {
						list.remove(nameInSet);
						PeaControl.setAvailableFileNames(list);
					}
					// remove set from model:
					if (removedFromSet == false) {
						removedFromSet = NotebookModel.removeFromMap(nameInSet);
					}
				}
			}
			
			try{
				// remove DocumentListener:
				textPane.getDocument().removeDocumentListener(this);
				
				// remove Cut-Copy-Paste.Menue: 
				if (newFocusTab != null) {
					textPane.removeMouseListener(newFocusTab);
				}

				// remove redo and undo manager:
				textPane.getDocument().removeUndoableEditListener(manager);
				   
				//remove keyStroke: STRG + R, STRG + W
				textPane.unregisterKeyboardAction(KeyStroke.getKeyStroke(
				            KeyEvent.VK_R, InputEvent.CTRL_DOWN_MASK ));
				textPane.unregisterKeyboardAction(KeyStroke.getKeyStroke(
				            KeyEvent.VK_W, InputEvent.CTRL_DOWN_MASK ));	
			} catch (Exception e) {
				TestLog.e(getClass(), e.getMessage());
			}

			// delete the file name: file should never be overwritten
			newFocusTab.setFileName(null);
			
			// get the index of the new tab to set the focus:
			int index = tabPane.indexOfComponent(lastFocusTab);

			if (index != -1) { //  in tabPane
				// Handle new selection of tab first: if possible, set to lastFocusTab
				if (lastFocusTab != null 
						//&& lastFocusTab.equals(newFocusTab) == false
						&& (tabPane.indexOfComponent(lastFocusTab) != -1)
						&& lastFocusTab.getFileNameWithoutPath() != null) {
					tabPane.setSelectedComponent(lastFocusTab);
				} else {
					// set focus to first tab
					if (tabPane.getTabCount() > 0) {
						tabPane.setSelectedComponent(tabPane.getComponentAt(0));
					} else {
						PeaDialog.showMessage(this, PeaProperties.getVmBridge().translate("no_valid_file_found"), PeaProperties.getVmBridge().translate("error"), 0);
						return PeaProperties.getVmBridge().translate("no_valid_file_found");
					}
				}
				// remove the tab (which is not selected now)
				if (newFocusTab != null &&
						tabPane.getTabCount() > 0 && 
						(tabPane.indexOfComponent(newFocusTab) != -1)){
					try { // if there is no valid tab left, this will throw an IndexOutOfBoundException
						tabPane.remove(newFocusTab);
					} catch (IndexOutOfBoundsException e) {
						TestLog.e(getClass(), e.toString() + 
								", removing tab failed: " + newFocusTab.getFileNameWithoutPath()
								+ ", " + e.getMessage());
					}
					
				} 
			} else {

				// remove the tab (which is not selected now)
				if (newFocusTab != null &&
						tabPane.getTabCount() > 0 && 
						(tabPane.indexOfComponent(newFocusTab) != -1)){
					try { // if there is no valid tab left, this will throw an IndexOutOfBoundException
						tabPane.remove(newFocusTab);
					} catch (IndexOutOfBoundsException e) {
						TestLog.e(getClass(), e.toString() + 
								", removing tab failed: " + newFocusTab.getFileNameWithoutPath()
								+ ", " + e.getMessage());
					}					
				} 
				// return an error message to show that an error appeared
				// this is neccesary, when the change listener is not yet registered in setChangeListener
				return ("lastFocusTab not in tabPane");
			}
		}
		
		// Cut-Copy-Paste.Menue: 
		textPane.addMouseListener(newFocusTab);
		// Undo/Redo: key and menu
		manager = new UndoManager();
		UndoAction undoAction = new UndoAction(manager);
		RedoAction redoAction = new RedoAction(manager);
		textPane.getDocument().addUndoableEditListener(manager);
		   
		//keyStroke: STRG + R, STRG + W
		textPane.registerKeyboardAction(undoAction, KeyStroke.getKeyStroke(
		            KeyEvent.VK_R, InputEvent.CTRL_DOWN_MASK ), JComponent.WHEN_FOCUSED);
		textPane.registerKeyboardAction(redoAction, KeyStroke.getKeyStroke(
		            KeyEvent.VK_W, InputEvent.CTRL_DOWN_MASK ), JComponent.WHEN_FOCUSED);
		

		// add listener
		textPane.getDocument().addDocumentListener(this);		

		// set tab title with pea icon and green background
		newFocusTab.setTabTitle(true); // true = icon, green

		if (newFocusTabFailed == false) {
			// set the component as lastFocusTab
			lastFocusTab = newFocusTab;
		}

		// DocumentListener should not fire event for text setting
		docChangeUnsaved = false;
		
		return null;
	}
	
	
	//-------------------------override-----------------------------
	// ChangeListener: event if selection changes: 
	// there are some cases where getSelectedIndex() 
	// and getSelectedComponen() are not the same!!!
	@Override
	public void stateChanged(ChangeEvent ce) {	
		
		try { 
			// set the wait cursor: 
			this.setCursor(CursorManager.getWaitCursor());

			if (tabPane.getSelectedComponent() == null) {
				// no tab selected
				if (tabPane.getTabCount() > 0){
						//&& tabPane.getComponent(0) instanceof TabComponent) { // not TabContainer
					// set focus on first tab
					TabComponent newFocus = (TabComponent) tabPane.getComponentAt(0);
					if (newFocus != null &&
							tabPane.indexOfComponent(newFocus) != -1){
						tabPane.setSelectedComponent(newFocus);
					}

				} else {
					// no tab
					TestLog.v(LockFrameEditor.class, "No tab to set focus...");				
				}
				docChangeUnsaved = false;
			} else {
				changeFocus((TabComponent) tabPane.getSelectedComponent());
			}						
		} finally {
			// reset the cursor:
	    	this.setCursor(CursorManager.getDefaultCursor());
		} 
	}
	
	// DocumentListener
	@Override
	public void changedUpdate(DocumentEvent de) {
		if (lastFocusTab.getTextPane().getSelectedText() != null) {
			// only when there is selected text, otherwise this fires always
			docChangeUnsaved = true;
		}
	}
	@Override
	public void insertUpdate(DocumentEvent arg0) {
		docChangeUnsaved = true;	
	}
	@Override
	public void removeUpdate(DocumentEvent arg0) {
		docChangeUnsaved = true;
	}
	
	// ContentUtilities
	public void windowClosingCommands() {
		
		// Check if algorithms or parameters were changed
		// but not yet applied: 
		if (isChangesApplied() == false) {
			changePasswordOrApplyChanges(frame, true);
			setChangesApplied(true);
		}
		
		// ask to save unsaved modifications of the text
		if (checkUnsavedContentChanges() == true) {
			// do not check twice
			docChangeUnsaved = false;
			// if cloud file is to be saved:
			// doInBackground must set running process before
			// checking if processes are running, 
			// otherwise changes may not be saved
			try {
				Thread.sleep(50);
			} catch (InterruptedException e) {
				// TODO Auto-generated catch block
				TestLog.e(LockFrameEditor.class, "Sleep thread interrupted...");
				//e.printStackTrace();
			}
		}	

		// check if all files are in path file and ask to store if not
		HashSet<String> availableSet = new HashSet<String>(PeaControl.getAvailableFileNames());
		// add files from model:
		ArrayList <HashSet<String>> setList = NotebookModel.getSetList();
		for (HashSet<String> setInList : setList) {
			availableSet.addAll(setInList);
		}
		if (availableSet.size() > 0) {
			PathFileManager.addFileNamesToPathFile(availableSet);
		}
		
		// save properties
		new PropertyHandler().updateAllPropValues();				
		
		// check if cloud processes are still running and ask to exit
		if (CloudControl.getNumberOfRunningCloudProcesses() > 0) {
			String[] options = {PeaProperties.getVmBridge().translate("cancel"),
					PeaProperties.getVmBridge().translate("ok")};
					
			
			int result = PeaDialog.showOptionYAxis(this, 
					PeaProperties.getVmBridge().translate("cloud_process_warning"), 
					PeaProperties.getVmBridge().translate("warning"), 
					2,  options, 0);

			if (result == 1) {
				TestLog.v(this.getClass(), "Closing although cloud process is running...");
			} else {
				TestLog.v(this.getClass(), "Upload process is running.... Break exit...");
				return;
			}
		}	

		// clear the text
		if (tabPane.getSelectedComponent() != null) {
			StyledDocument doc = ((TabComponent) tabPane.getSelectedComponent()).getTextPane().getStyledDocument();
			try {
				doc.remove(0, doc.getLength() );
			} catch (BadLocationException e) {
				TestLog.e(LockFrameEditor.class, "LockFrameCalendar: " + e + ", " + e.getLocalizedMessage());
				//e.printStackTrace();
			}			
		}

		// clear the sessionKeys:
		CipherStuff.getSessionKeyCrypt().clearKeys();	
		
		// Close all providers
		Collection<StorageProvider> openProviders = CloudControl.getOpenProviders();
		if (openProviders != null &&  ! openProviders.isEmpty()) {
			for (StorageProvider sp : openProviders) {
				String error = sp.closeSession();
				if (error != null) {
					TestLog.e(getClass(), error);
				}
			}
		}
		
		// delete all temporary backup files of cloud files:
		if (dialog.getEncryptedFileTypePanel() != null) { // after initializing maybe null
			CloudFilePanel cfp = dialog.getEncryptedFileTypePanel().getCloudFilePanel();
			if (cfp != null) {
				String[] providers = cfp.getCloudControl().getUsedProviders();
				if (providers != null) {
					for (String provName : providers) {	
						if (provName != null) {
							String[] filesOfProv = cfp.getCloudControl().getUsedFileNamesOfProvider(provName);
							if (filesOfProv != null) {
								for (String fileName : filesOfProv) {
									if (fileName != null && fileName.length() >0) {
										File fileToDelete = new File(CloudControl.getTmpFileNameForCloudFile(fileName, provName, null));
										if (fileToDelete.exists()) {
											boolean deleteResult = fileToDelete.delete();
											if (deleteResult == false) {
												TestLog.e(LockFrameEditor.class, "Could not delete " + fileToDelete);
											} else {
											}
										} else {
											TestLog.e(LockFrameEditor.class, "File form cloud to delete does not exist " + fileToDelete);
										}
									} else {
										TestLog.e(LockFrameEditor.class, "Could find file to delete from cloud " + provName);
									}
								}
							} else {
								TestLog.e(LockFrameEditor.class, "Could find file to delete from cloud... ");
							}
						} else {
							TestLog.e(LockFrameEditor.class, "Cloud provider is null... ");
						}
					}
				}
			}
		}
		TestLog.o(this.getClass(), "Exit");
		System.exit(0);			
	}

	/**
	 * Get plain text as byte array 
	 * of current text pane
	 * 
	 * @return	the plain text as byte array
	 */
	public byte[] getPlainBytes() {

		JTextPane textPane = lastFocusTab.getTextPane();//((TabComponent) tabPane.getSelectedComponent()).getTextPane();
		DefaultStyledDocument doc = (DefaultStyledDocument) textPane.getStyledDocument();
		if (doc == null) {
			TestLog.e(LockFrameEditor.class, "no document");
			return null;
		}
		byte[] plainBytes = null;

		// UndoListener can not be serialized.. 
		textPane.getDocument().removeUndoableEditListener(manager);
		plainBytes = LockFrameEditor.serialize(doc);
	    textPane.getDocument().addUndoableEditListener(manager);
		textPane.getDocument().addDocumentListener(this);

		return plainBytes;
	}
	@Override
	public void windowClosing(WindowEvent arg0) { // by X-Button
		windowClosingCommands();
	}
	@Override
	public void windowActivated(WindowEvent arg0) {}
	@Override
	public void windowClosed(WindowEvent arg0) {
		//windowClosingCommands();
	}
	@Override
	public void windowDeactivated(WindowEvent arg0) {}
	@Override
	public void windowDeiconified(WindowEvent arg0) {}
	@Override
	public void windowIconified(WindowEvent arg0) {}
	@Override
	public void windowOpened(WindowEvent arg0) {}

	
	
	//--------------------------------------------------------------------------
	// Getter & setter
	
	/**
	 * Set the text for the currently selected tab
	 * as StyledDocument
	 * 
	 * @param textChars		the text as an array of characters
	 */
	protected void setChars( char[] textChars ) {

		//StyledDocument doc = textPane.getStyledDocument();
		StyledDocument doc = ((TabComponent) tabPane.getSelectedComponent()).getTextPane().getStyledDocument();
		try {
			doc.insertString(0, new String(textChars), null );
			docChangeUnsaved = false;
		} catch (BadLocationException e) {
			TestLog.e(LockFrameEditor.class, "LockFrameCalendar: " + e);
			e.printStackTrace();
		}		
	}

	/**
	 * Get the currently focused JTextPane 
	 * 
	 * @return	the currently focused JTextPane 
	 */
	protected static synchronized JTextPane getFocusedTextPane(){
		return lastFocusTab.getTextPane();
	}
	/**
	 * Get the currently selected tab
	 * 
	 * @return	the currently selected instance of TabComponent
	 */
	protected static synchronized TabComponent getCurrentTab(){
		if (lastFocusTab == null) {
			new UnexpectedValueException("TabComponent", "lastFocusTab", "is null").printDescription();
		}
		return lastFocusTab;
	}
	private static synchronized void setLastFocusTab(TabComponent c) {
		LockFrameEditor.lastFocusTab = c;
	}
	/**
	 * Get the file names without path of all currently available tabs
	 * 
	 * @return	the file names as an array of Strings
	 */
	protected static String[] getFileNames(){

		int len = tabPane.getTabCount();
		String[] fileNames = new String[len];
		for (int i = 0; i < len; i++) {
			fileNames[i] = ( (TabComponent) tabPane.getComponentAt(i)).getFileNameWithoutPath();
		}
		return fileNames;
	}

	/**
	 * Set value for unsaved changes of the currently
	 * selected tabs JTextPane. 
	 * 
	 * @param docChangeUnsaved the docChangeUnsaved to set
	 */
	public void setDocChangeUnsaved(boolean newDocChangeUnsaved) {
		this.docChangeUnsaved = newDocChangeUnsaved;
	}
	/**
	 * Get value for unsaved changes of the currently
	 * selected tabs JTextPane. 
	 * 
	 * @return true if there are unsaved changes of the current document
	 */
	public boolean isDocChangeUnsaved() {
		return this.docChangeUnsaved;
	}

	//----------------------------------------------------------------------------------------
	// classes: 
	
	// The Undo action
	private class UndoAction extends AbstractAction {
		private static final long serialVersionUID = 1L;

		public UndoAction(UndoManager manager) {
	      this.manager = manager;
	    }
	    public void actionPerformed(ActionEvent evt) {
	    	
	    	try {
	  	    	if (manager.canUndo()) {
	  	        manager.undo();
	  	    } else {
		    	JOptionPane.showMessageDialog(frame,
		    			"Cannot undo.");
		    	TestLog.e(LockFrameEditor.class, "LockFrameCalendar: undo action failed");
		    	}
	  	    } catch (CannotUndoException e) {
	  	        Toolkit.getDefaultToolkit().beep();
	  	    }
	    	
	    }
	    private UndoManager manager;
	}
	// The Redo action
	private class RedoAction extends AbstractAction {
		private static final long serialVersionUID = 1L;

		public RedoAction(UndoManager manager) {
	      this.manager = manager;
	    }
	    public void actionPerformed(ActionEvent evt) {
	      try {
	    	  if (manager.canRedo()){
	    		  manager.redo();
	    	  } else {
	    		  TestLog.e(LockFrameEditor.class, "LockFrameCalendar: redo action failed");
	    	  }
	      } catch (CannotRedoException e) {
	        Toolkit.getDefaultToolkit().beep();
	      }
	    }
	    private UndoManager manager;
	}
	
    public AccessibleContext getAccessibleContext() {
        if (accessibleContext == null) {
            accessibleContext = new AccessibleLockFrameEditor();
        }
        return accessibleContext;
    }

    protected class AccessibleLockFrameEditor extends AccessibleJFrame {
        //Inherit everything, override nothing.
    }


	@Override
	public String[] getCurrentFileNames() {
		return Converter.hashSetToArray(getCurrentTab().getFileNames());
	}
	@Override
	public void setMessage(String message, boolean isError) {
		if(isError == true) {
			messageLabel.setForeground(Color.red);
			messageLabel.setText(message);			
		} else {
			messageLabel.setForeground(Color.black);
			messageLabel.setText(message);
		}
	}

	@Override
	public String saveCurrentContentInCloudTempFile(String providerName, String fileNameToStore) {
		
		String orig = null;
		if ( ! fileNameToStore.contains(CloudControl.getTmpDir())) {
			orig = fileNameToStore;
		} 

		String tmpFileName = CloudControl.getTmpFileNameForCloudFile(new File(fileNameToStore).getName(), providerName, orig);
		//System.getProperty("java.io.tmpdir") 
		//+ File.separator + providerName + File.separator + fileNameToStore;
		File file = new File(tmpFileName);
		if (file.exists() == false) {
			try {
				file.getParentFile().mkdirs();
				file.createNewFile();
				TestLog.v(this.getClass(), "Temp file created for cloud " + providerName + ": " + file.getAbsolutePath());
			} catch (IOException e) {
				TestLog.e(getClass(), fileNameToStore + " - " + new ExceptionHelper(e).getInfos());
				return PeaProperties.getVmBridge().translate("error") + "\nCreate tmp file failed";
			}
		}
		
		if (docChangeUnsaved == false) {
			// copy ciphertext from existing file whether backup of cloud file or not
			byte[] cipherText = ReadResources.readExternFile(fileNameToStore);

			if (cipherText == null) {
				return PeaProperties.getVmBridge().translate("error") + "\n(read ciphertext)";
			}

			boolean success = ContentUtilities.saveContent(null, 
					null, cipherText,
					tmpFileName, true);
			if (success == false) {
				TestLog.e(getClass(), fileNameToStore + " - " + "Save calendar in temp file failed");
				return PeaProperties.getVmBridge().translate("error") + "\n(save ciphertext)";
			} else {
				//TestLog.v(this.getClass(), "Calendars saved in temp file " + tmpFileName);
				return null;
			}

		} else {
			byte[] plainBytes = ((LockFrameEditor) PeaControl.getDialog().getLockFrame()).getPlainBytes();
			// encrypt the calendars by session key and store in file
			boolean success = ContentUtilities.saveContent(CipherStuff.getSessionKeyCrypt().getKey(), 
					plainBytes, null,
					tmpFileName, true);
			if (success == false) {
				TestLog.e(getClass(), fileNameToStore + " - " + "Save calendar in temp file failed");
				return PeaProperties.getVmBridge().translate("error") + "\n" + CipherStuff.getErrorMessage();
			} else {
				//TestLog.v(this.getClass(), "Calendars saved in temp file " + tmpFileName);
				return null;
			}
		}

	}
	@Override
	public void setCurrentFileNames(String[] fileNameToSet) {
		new UnexpectedValueException("function","setCurrentFileNames","is not implemented").printDescription();
	}
	@Override
	public void updateModelWithCloudFiles(File[] files, String providerName) {

		for (File f : files) {
			//String fileNameWithoutPath = f.getName();
			String tmpFileName = CloudControl.getTmpFileNameForCloudFile(f.getName(), providerName, f.getAbsolutePath());
			//boolean added = 
			NotebookModel.addFileNameToSet(tmpFileName,f.getAbsolutePath());
//TestLog.v(this.getClass(), "Added " + tmpFileName + " : " + added);
		}
	}
	@Override
	public String decryptAndLoadFileToView(String fileName) {
		
		if (fileName == null) {
			new UnexpectedValueException("String", "fileName", "is null").printDescription();
		}
		boolean newTab = false;
		HashSet<String> tabFileNames = NotebookModel.getFileNameSet(fileName);
				//NotebookModel.getFileNames(fileName);
		if (tabFileNames == null || tabFileNames.isEmpty()) { // Download file from cloud
			//new UnexpectedValueException("HashSet<String>", "tabFileNames", "is null or empty for: "+ fileName).printDescription();
			HashSet<String> set = new HashSet<String>();
			set.add(fileName);
			NotebookModel.addFileNameSet(set);
			tabFileNames = set;
			newTab = true;
			// add to CloudFileModel:
			if (fileName.contains(CloudControl.getTmpDir())) {
				dialog.getEncryptedFileTypePanel().getCloudFilePanel().getCloudControl().getCloudFileModel().addUsedFileToProvider(
						CloudControl.getProviderNameFromTmpFile(fileName), 
						CloudControl.getFileNameFromTmpFile(fileName));
			}  			
		}
		// create tab
		createNewTab(tabFileNames);
		// set focus on new tab
		if (newTab == true) {
			Component c = tabPane.getComponent(tabPane.getComponentCount() -1);
			if (c instanceof TabComponent) {
				tabPane.setSelectedComponent(c);
				//changeFocus((TabComponent) c);
			}
		}
		return null;// no error occurred
	}
	/**
	 * Get the number of Tabs shown in the JTabbedPane.
	 * 
	 * @return the number of shown tabs
	 */
	protected int getNumberOfTabs() {
		return tabPane.getTabCount();
	}
	
	//===========================================
	// DOCUMENTS AND BYTE ARRAYS
	/**
	 * Serialize a DefaultStyledDocument with a RTFEditorKit. 
	 * Note: Till Java 9, the alignment property is not saved.
	 * 
	 * @param dsd	the document
	 * 
	 * @return		the document as an array of bytes
	 */
	public final static byte[] serialize(DefaultStyledDocument dsd)  {

		RTFEditorKit kit =  new RTFEditorKit();
		ByteArrayOutputStream out = new ByteArrayOutputStream();
		try {
			kit.write( out,  dsd, 0, dsd.getLength() );
			out.close();
		} catch (BadLocationException e) {
			TestLog.e(LockFrameEditor.class, "Converter: " + e);
			e.printStackTrace();
		} catch (IOException e) {
			TestLog.e(LockFrameEditor.class, "Converter: " + e);
			e.printStackTrace();
		}  
	    return out.toByteArray();
	}
	/**
	 * Convert a serialized DefaultStyledDocument as an array
	 * of bytes into a DefaultStyledDocument. 
	 * Note: Till Java 9, the alignment property is not included.
	 * 
	 * @param data	the serialized document as an array of bytes
	 * 
	 * @return		a DefaultStyledDocument
	 */
	public final static DefaultStyledDocument deserialize(byte[] data)  {
		RTFEditorKit kit =  new RTFEditorKit();
		DefaultStyledDocument dsd = new DefaultStyledDocument();
		ByteArrayInputStream in = new ByteArrayInputStream(data);
		try {
			kit.read(in, dsd, 0);
			in.close();
		} catch (BadLocationException e) {
			// 
			TestLog.e(LockFrameEditor.class, "Converter deserialize:  "+ e.toString());
			return null;
			//e.printStackTrace();
			//TestLog.e(LockFrameEditor.class, "BadLocationException");
		} catch (IOException e) {

			TestLog.e(LockFrameEditor.class, "Converter deserialize:  "+ e.toString());
			return null;
			//e.printStackTrace();
		}		
	    return dsd;
	}
	@Override
	public String saveCurrentContent() {
		try {
			HashSet<String> tabSet = lastFocusTab.getFileNames();
			byte[] plainBytes = getPlainBytes();
			if (plainBytes == null) {
				TestLog.e(getClass(), "No plaintext found ");
				TestLog.ve(getClass(), "No plaintext found ", 5);
				return PeaProperties.getVmBridge().translate("unexpected_error") + " (no plaintext)";
			}
			byte[] cipherBytes = CipherStuff.getInstance().encrypt(plainBytes, // is zeroized
					CipherStuff.getSessionKeyCrypt().getKey(), // keyMaterial is overwritten
					true, null ); // key is stored encrypted
			if (cipherBytes == null) {
				TestLog.e(getClass(), "No ciphertext found ");
				TestLog.ve(getClass(), "No ciphertext found ", 5);
				return PeaProperties.getVmBridge().translate("decryption_failed");
			}
			boolean cloudFileToSave = false;
			for (String nameInTab : tabSet) {					
				boolean success = ContentUtilities.saveContent(null, null, cipherBytes, nameInTab, true);
				if (success == false) {
					TestLog.e(getClass(), "Saving content failed ");
					TestLog.ve(getClass(), "Saving content failed " , 5);
				}
				if (nameInTab.contains(CloudControl.getTmpDir())) {
					cloudFileToSave = true;
				}
			}
			// if cloud files: wait until all UploadWorkers started
			// and set running process....
			if (cloudFileToSave == true) {
				// wait a short time, because all UploadWorkers
				// must have initialized and set running processes
				try {
					Thread.sleep(50);
				} catch (InterruptedException e) {
					//e.printStackTrace();
				}
			}
			docChangeUnsaved = false;
		} catch (Exception e) {
			TestLog.e(getClass(), "Saving text failed: " + e.getLocalizedMessage() + " - " + e.toString());
			TestLog.ve(getClass(), "Saving text failed: " + e.toString(),  5);
			return e.getLocalizedMessage();
		}

		this.setDocChangeUnsaved(false);
		return null;
	}



	
/*	protected void changeFocusToFirstTab() {
		int tabCount = tabPane.getTabCount();
		TabComponent tab = null;
		for (int i = 0; i < tabCount; i++) {
			if (tabPane.getTabComponentAt(i) instanceof TabComponent) {
				tab = (TabComponent) tabPane.getTabComponentAt(i);
				break;
			} 
//else {TestLog.e(getClass(), i + " not tab");}
		}
		if (tab == null) {
			new UnexpectedValueException("TabComponent", "tab","is null").printDescription();
		}
		changeFocus(tab);
	}*/
	
	
	/**
	 * Get all tabs of the JTabbedPane
	 * 
	 * @return	the tabs of the tabPane
	 */
/*	protected TabComponent[] getTabs() {
		int tabCount = tabPane.getTabCount();
		ArrayList<TabComponent> list = new ArrayList<TabComponent>();
		for (int i = 0; i< tabCount; i++) {
			list.add((TabComponent) tabPane.getTabComponentAt(i));
		}
		return list.toArray(new TabComponent[0]);
	}*/
}