package cologne.eck.note_pea;

/*
 * Peafactory - Production of Password Encryption Archives
 * Copyright (C) 2015  Axel von dem Bruch
 * 
 * This library is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU General Public License as published 
 * by the Free Software Foundation; either version 2 of the License, 
 * or (at your option) any later version.
 * This library is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * See:  http://www.gnu.org/licenses/gpl-2.0.html
 * You should have received a copy of the GNU General Public License 
 * along with this library.
 */

/**
 * Menu of main frame of editor pea.  
 */

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;
import java.util.Arrays;
import java.util.HashSet;

import javax.accessibility.AccessibleContext;
import javax.swing.*;
import javax.swing.filechooser.FileFilter;
import javax.swing.text.*;
import javax.swing.text.rtf.RTFEditorKit;

import cologne.eck.all_peas.cloud.CloudControl;
import cologne.eck.all_peas.cloud.gui.CloudMenu;
import cologne.eck.all_peas.control.PeaControl;
import cologne.eck.all_peas.data.Attachments;
import cologne.eck.all_peas.data.PeaProperties;
import cologne.eck.all_peas.gui.CursorManager;
import cologne.eck.all_peas.gui.PeaDialog;
import cologne.eck.all_peas.gui.PeaFileChooser;
import cologne.eck.all_peas.gui.PswDialogView;
import cologne.eck.all_peas.gui.menu.HelpMenu;
import cologne.eck.all_peas.gui.menu.PeaMenuBar;
import cologne.eck.all_peas.gui.menu.SettingMenu;
import cologne.eck.tools.Converter;
import cologne.eck.tools.ExceptionHelper;
import cologne.eck.tools.ReadResources;
import cologne.eck.tools.TestLog;
import cologne.eck.tools.UnexpectedValueException;
import cologne.eck.tools.WriteResources;
import cologne.eck.tools.Zeroizer;
import settings.PeaSettings;

@SuppressWarnings("serial")
final class LockFrameEditorMenu extends PeaMenuBar implements ActionListener {

	// private static ResourceBundle PeaProperties.getBundle();
	private static LockFrameEditor lockFrame;// = (JFrame) PeaControl.getDialog().getLockFrame();

	private static CloudMenu cloudMenu;
	
	public LockFrameEditorMenu(LockFrameEditor _lockFrame) {

		// PeaProperties.getBundle() = PeaProperties.getBundle();
		// setBorder(new EtchedBorder() );
		
		

		if (_lockFrame == null) {
			lockFrame = (LockFrameEditor) PeaControl.getDialog().getLockFrame();
			if (lockFrame == null) {
				TestLog.ve(this.getClass(), "Missing lockFrame ", 5);
			}
		} else {
			lockFrame = _lockFrame;
		}
		int fontSize = PeaProperties.getFontSize();//PswDialogView.getFontsize();
		Font buttonFont = new Font(Font.SANS_SERIF, Font.PLAIN, fontSize);

		JMenu fileMenu = new JMenu(PeaProperties.getVmBridge().translate("file"));// File");//\u00FC");
		fileMenu.setFont(buttonFont);
		// fileMenu.setMnemonic(KeyEvent.VK_F);
		fileMenu.getAccessibleContext().
	    setAccessibleDescription(
	    	    "Clicking this component opens menu to file operations: save, open, change password, create random password,quit");
		fileMenu.setMnemonic(KeyStroke.getKeyStroke(PeaProperties.getVmBridge().translate("file_mnemonic")).getKeyCode());
		this.add(fileMenu);

		JMenuItem saveItem = new JMenuItem(PeaProperties.getVmBridge().translate("save_encrypted"));// save (encrypted)");
		saveItem.setActionCommand("save");
		saveItem.setFont(buttonFont);
		saveItem.getAccessibleContext().
	    setAccessibleDescription(
	    	    "Clicking this component saves the current document.");
		saveItem.setMnemonic(KeyStroke.getKeyStroke(PeaProperties.getVmBridge().translate("save_mnemonic")).getKeyCode());
		saveItem.addActionListener(this);
		fileMenu.add(saveItem);

		JMenuItem exportItem = new JMenuItem(PeaProperties.getVmBridge().translate("save_as"));// "Save as...");
		exportItem.setActionCommand("export");
		exportItem.getAccessibleContext().
	    setAccessibleDescription(
	    	    "Clicking this component saves the document with custom name.");
		exportItem.setFont(buttonFont);
		exportItem.setMnemonic(
				KeyStroke.getKeyStroke(PeaProperties.getVmBridge().translate("save_as_mnemonic")).getKeyCode());
		exportItem.addActionListener(this);
		fileMenu.add(exportItem);
/* TODO		
		JMenuItem dropboxStoreItem = new JMenuItem(PeaProperties.getOsBridge().translate("dropbox_store"));// "Save as...");
		dropboxStoreItem.setActionCommand("dropboxStore");
		dropboxStoreItem.getAccessibleContext().
	    setAccessibleDescription(
	    	    "Clicking this component uploads the current document to Dropbox.");
		dropboxStoreItem.setFont(buttonFont);
		dropboxStoreItem.setMnemonic(
				KeyStroke.getKeyStroke("D").getKeyCode());
		dropboxStoreItem.addActionListener(this);
		fileMenu.add(dropboxStoreItem);		*/

		JMenuItem openItem = new JMenuItem(PeaProperties.getVmBridge().translate("import_file"));// "open file (txt, rtf or lock file)");
		openItem.getAccessibleContext().
	    setAccessibleDescription(
	    	    "Clicking this component imports another text file.");
		openItem.setActionCommand("open");
		openItem.setFont(buttonFont);
		openItem.setMnemonic(
				KeyStroke.getKeyStroke(PeaProperties.getVmBridge().translate("import_mnemonic")).getKeyCode());
		openItem.addActionListener(this);
		fileMenu.add(openItem);

		if (PeaSettings.getPswGenerator() != null) {
			JMenuItem randomPswItem = new JMenuItem(PeaProperties.getVmBridge().translate("generate_random_password"));// "Generate
			randomPswItem.getAccessibleContext().
		    setAccessibleDescription(
		    	    "Clicking this component creates a radom password in a text field.");																										
			randomPswItem.setMnemonic(
					KeyStroke.getKeyStroke(PeaProperties.getVmBridge().translate("random_psw_mnemonic")).getKeyCode());
			randomPswItem.addActionListener(this);
			randomPswItem.setActionCommand("randomPsw");
			fileMenu.add(randomPswItem);
		}

		JMenuItem pswItem = new JMenuItem(PeaProperties.getVmBridge().translate("change_password"));// "Change password");
		pswItem.setActionCommand("psw");
		pswItem.getAccessibleContext().
	    setAccessibleDescription(
	    	    "Clicking this component opens dialog to change the password.");
		pswItem.setFont(buttonFont);
		pswItem.setMnemonic(
				KeyStroke.getKeyStroke(PeaProperties.getVmBridge().translate("change_psw_mnemonic")).getKeyCode());
		pswItem.addActionListener(this);
		fileMenu.add(pswItem);

		JMenuItem quitItem = new JMenuItem(PeaProperties.getVmBridge().translate("quit"));// "Quit");
		quitItem.setActionCommand("quit");
		quitItem.getAccessibleContext().
	    setAccessibleDescription(
	    	    "Clicking this component exits the program.");
		quitItem.setFont(buttonFont);
		quitItem.setMnemonic(KeyStroke.getKeyStroke(PeaProperties.getVmBridge().translate("quit_mnemonic")).getKeyCode());
		quitItem.addActionListener(this);
		fileMenu.add(quitItem);

		JMenu editMenu = new JMenu(PeaProperties.getVmBridge().translate("edit"));// "Edit");
		editMenu.getAccessibleContext().
	    setAccessibleDescription(
	    	    "Clicking this component opens edit menu: redo, undo, cut, copy paste, select and default settings.");
		editMenu.setFont(buttonFont);
		editMenu.setMnemonic(KeyStroke.getKeyStroke(PeaProperties.getVmBridge().translate("edit_mnemonic")).getKeyCode());
		this.add(editMenu);

		JMenuItem undoItem = new JMenuItem(PeaProperties.getVmBridge().translate("undo"));// "Undo");
		undoItem.getAccessibleContext().
	    setAccessibleDescription(
	    	    "Clicking this component causes undoing the last action.");
		undoItem.setActionCommand("undo");
		undoItem.setFont(buttonFont);
		undoItem.setMnemonic(KeyStroke.getKeyStroke(PeaProperties.getVmBridge().translate("undo_mnemonic")).getKeyCode());
		undoItem.addActionListener(this);
		editMenu.add(undoItem);

		JMenuItem redoItem = new JMenuItem(PeaProperties.getVmBridge().translate("redo"));// "Redo");
		redoItem.getAccessibleContext().
	    setAccessibleDescription(
	    	    "Clicking this component causes redoing the last action.");
		redoItem.setActionCommand("redo");
		redoItem.setFont(buttonFont);
		redoItem.setMnemonic(KeyStroke.getKeyStroke(PeaProperties.getVmBridge().translate("redo_mnemonic")).getKeyCode());
		redoItem.addActionListener(this);
		editMenu.add(redoItem);
		

		JMenuItem searchItem = new JMenuItem(PeaProperties.getVmBridge().translate("search"));// "Redo");
		searchItem.getAccessibleContext().
	    setAccessibleDescription(
	    	    "Clicking this component opens a dialog to insert a string to search for.");
		searchItem.setActionCommand("search");
		searchItem.setFont(buttonFont);
		searchItem.setMnemonic(KeyStroke.getKeyStroke(PeaProperties.getVmBridge().translate("search_mnemonic")).getKeyCode());
		searchItem.addActionListener(this);
		editMenu.add(searchItem);

		JMenuItem cutItem = new JMenuItem(PeaProperties.getVmBridge().translate("cut"));// "Cut");
		cutItem.getAccessibleContext().
	    setAccessibleDescription(
	    	    "Clicking this component cuts selected text.");
		cutItem.setActionCommand("cut");
		cutItem.setFont(buttonFont);
		cutItem.setMnemonic(KeyStroke.getKeyStroke(PeaProperties.getVmBridge().translate("cut_mnemonic")).getKeyCode());
		cutItem.addActionListener(this);
		editMenu.add(cutItem);

		JMenuItem copyItem = new JMenuItem(PeaProperties.getVmBridge().translate("copy"));// "Copy");
		copyItem.getAccessibleContext().
	    setAccessibleDescription(
	    	    "Clicking this component copies selected text.");
		copyItem.setActionCommand("copy");
		copyItem.setFont(buttonFont);
		copyItem.setMnemonic(KeyStroke.getKeyStroke(PeaProperties.getVmBridge().translate("copy_mnemonic")).getKeyCode());
		copyItem.addActionListener(this);
		editMenu.add(copyItem);

		JMenuItem pasteItem = new JMenuItem(PeaProperties.getVmBridge().translate("paste"));// "Paste");
		pasteItem.setActionCommand("paste");
		pasteItem.getAccessibleContext().
	    setAccessibleDescription(
	    	    "Clicking this component pastes text from clipboard.");
		pasteItem.setFont(buttonFont);
		pasteItem.setMnemonic(
				KeyStroke.getKeyStroke(PeaProperties.getVmBridge().translate("paste_mnemonic")).getKeyCode());
		pasteItem.addActionListener(this);
		editMenu.add(pasteItem);

		JMenuItem selectAllItem = new JMenuItem(PeaProperties.getVmBridge().translate("select_all"));// "Select All");
		selectAllItem.setActionCommand("selectAll");
		selectAllItem.getAccessibleContext().
	    setAccessibleDescription(
	    	    "Clicking this component selects all of the text.");
		selectAllItem.setFont(buttonFont);
		selectAllItem.setMnemonic(
				KeyStroke.getKeyStroke(PeaProperties.getVmBridge().translate("select_all_mnemonic")).getKeyCode());
		selectAllItem.addActionListener(this);
		editMenu.add(selectAllItem);
				
		
		JMenu settingMenu = new JMenu(PeaProperties.getVmBridge().translate("text_setting"));
		settingMenu.getAccessibleContext().
	    setAccessibleDescription(
	    	    "Clicking this component opens menu to set default font color, size and family and background color.");
		settingMenu.setFont(buttonFont);
		settingMenu.setMnemonic(
				KeyStroke.getKeyStroke(PeaProperties.getVmBridge().translate("text_setting_mnemonic")).getKeyCode());
		editMenu.add(settingMenu);
		
		JMenuItem defaultColorItem = new JMenuItem(PeaProperties.getVmBridge().translate("default_font_color"));// "Select All");
		defaultColorItem.getAccessibleContext().
	    setAccessibleDescription(
	    	    "Clicking this component opens a dialog to change the default color of the font.");
		defaultColorItem.setActionCommand("defaultFontColor");
		defaultColorItem.setFont(buttonFont);
		defaultColorItem.addActionListener(this);
		settingMenu.add(defaultColorItem);		
		
		JMenuItem defaultBackgroundItem = new JMenuItem(PeaProperties.getVmBridge().translate("default_background"));// "Select All");
		defaultBackgroundItem.setActionCommand("defaultBackground");
		defaultBackgroundItem.getAccessibleContext().
	    setAccessibleDescription(
	    	    "Clicking this component opens dialog to change the background of the text area.");
		defaultBackgroundItem.setFont(buttonFont);
		defaultBackgroundItem.addActionListener(this);
		settingMenu.add(defaultBackgroundItem);	
		
		JMenu defaultFamilyItem = new JMenu(PeaProperties.getVmBridge().translate("default_font_family"));// "Select All");
		defaultFamilyItem.getAccessibleContext().
	    setAccessibleDescription(
	    	    "Clicking this component opens a menu to choose the default font family: serif, sans serif, mono.");
		defaultFamilyItem.setFont(buttonFont);
		settingMenu.add(defaultFamilyItem);
		
		String fontNames[] = { "SERIF", "SANS_SERIF", "MONOSPACED" };
		for (String font : fontNames) {
			JMenuItem familyItem = new JMenuItem(font);
			familyItem.setFont(buttonFont);
			familyItem.setActionCommand("defaultFontFamily" + font);
			familyItem.addActionListener(this);
			defaultFamilyItem.add(familyItem);
		}
		
		JMenu defaultSizeItem = new JMenu(PeaProperties.getVmBridge().translate("default_font_size"));// "Select All");
		defaultSizeItem.setFont(buttonFont);
		defaultSizeItem.getAccessibleContext().
	    setAccessibleDescription(
	    	    "Clicking this component opens a menu to choose the font size from 6 to 24.");
		settingMenu.add(defaultSizeItem);	

		for (int i = 6; i < 25; i++) {
			JMenuItem sizeItem = new JMenuItem("" + i);
			sizeItem.setFont(buttonFont);
			sizeItem.setActionCommand("defaultFontSize" + i);
			sizeItem.addActionListener(this);
			defaultSizeItem.add(sizeItem);
		}	
		
		
		JMenu formatMenu = new JMenu(PeaProperties.getVmBridge().translate("format"));// "Format");
		formatMenu.setFont(buttonFont);
		formatMenu.getAccessibleContext().
	    setAccessibleDescription(
	    	    "Clicking this component opens menu to change the format of a selected text.");
		formatMenu.setMnemonic(
				KeyStroke.getKeyStroke(PeaProperties.getVmBridge().translate("format_mnemonic")).getKeyCode());
		this.add(formatMenu);

		JMenu fontMenu = new JMenu(PeaProperties.getVmBridge().translate("font"));// "Font");
		fontMenu.setFont(buttonFont);
		fontMenu.getAccessibleContext().
	    setAccessibleDescription(
	    	    "Clicking this component opens a menu to choose the font family: serif, sans serif, mono.");
		fontMenu.setMnemonic(KeyStroke.getKeyStroke(PeaProperties.getVmBridge().translate("font_mnemonic")).getKeyCode());
		formatMenu.add(fontMenu);

		//String fontNames[] = { "SERIF", "SANS_SERIF", "MONOSPACED" };
		for (String font : fontNames) {
			fontMenu.add(new StyledEditorKit.FontFamilyAction(font, font));
		}

		JMenu fontStyleMenu = new JMenu(PeaProperties.getVmBridge().translate("font_style"));// "Font style");
		fontStyleMenu.setFont(buttonFont);
		fontStyleMenu.getAccessibleContext().
	    setAccessibleDescription(
	    	    "Clicking this component opens a menu to choose the font style: bold,italic, underline.");
		fontStyleMenu.setMnemonic(
				KeyStroke.getKeyStroke(PeaProperties.getVmBridge().translate("style_mnemonic")).getKeyCode());
		formatMenu.add(fontStyleMenu);

		Action action = new StyledEditorKit.BoldAction();
		action.putValue(Action.NAME, PeaProperties.getVmBridge().translate("bold"));// "Bold");
		fontStyleMenu.add(action);

		action = new StyledEditorKit.ItalicAction();
		action.putValue(Action.NAME, PeaProperties.getVmBridge().translate("italic"));// "Italic");
		fontStyleMenu.add(action);

		action = new StyledEditorKit.UnderlineAction();
		action.putValue(Action.NAME, PeaProperties.getVmBridge().translate("underline"));// "Underline");
		fontStyleMenu.add(action);

		JMenu alignmentMenu = new JMenu(PeaProperties.getVmBridge().translate("alignment"));// "Font style");
		alignmentMenu.setFont(buttonFont);
		alignmentMenu.getAccessibleContext().
	    setAccessibleDescription(
	    	    "Clicking this component opens a menu to change the text alignment: left, center, right.");
		alignmentMenu.setMnemonic(
				KeyStroke.getKeyStroke(PeaProperties.getVmBridge().translate("alignment_mnemonic")).getKeyCode());
		formatMenu.add(alignmentMenu);

		Action alignmentAction = new StyledEditorKit.AlignmentAction(PeaProperties.getVmBridge().translate("left"), 0);
		alignmentMenu.add(alignmentAction);

		alignmentAction = new StyledEditorKit.AlignmentAction(PeaProperties.getVmBridge().translate("center"), 1);
		alignmentMenu.add(alignmentAction);

		alignmentAction = new StyledEditorKit.AlignmentAction(PeaProperties.getVmBridge().translate("right"), 2);
		alignmentMenu.add(alignmentAction);

		JMenu fontSizeMenu = new JMenu(PeaProperties.getVmBridge().translate("font_size"));// "Font size");
		fontSizeMenu.getAccessibleContext().
	    setAccessibleDescription(
	    	    "Clicking this component opens a menu to choose the font size from 6 to 24.");
		fontSizeMenu.setFont(buttonFont);
		fontSizeMenu
				.setMnemonic(KeyStroke.getKeyStroke(PeaProperties.getVmBridge().translate("size_mnemonic")).getKeyCode());
		formatMenu.add(fontSizeMenu);

		for (int i = 6; i < 25; i++) {
			fontSizeMenu.add(new StyledEditorKit.FontSizeAction("" + i, i));
		}

		JMenuItem fontColorMenu = new JMenuItem(PeaProperties.getVmBridge().translate("font_color"));
		fontColorMenu.getAccessibleContext().
	    setAccessibleDescription(
	    	    "Clicking this component opens a dialog to choose the font color for a selected text.");
		fontColorMenu.setFont(buttonFont);
		fontColorMenu.setMnemonic(
				KeyStroke.getKeyStroke(PeaProperties.getVmBridge().translate("color_mnemonic")).getKeyCode());
		fontColorMenu.addActionListener(this);
		fontColorMenu.setActionCommand("fontColor");
		formatMenu.add(fontColorMenu);
		
		cloudMenu = new CloudMenu(lockFrame);
		this.add(cloudMenu);
		
		JMenu algoSettingMenu = new SettingMenu(lockFrame);
		this.add(algoSettingMenu);

		JMenu helpMenu = new HelpMenu(lockFrame);		
		this.add(helpMenu);
		

		this.setMaximumSize(this.getPreferredSize());
		
	}

	@Override
	public void actionPerformed(ActionEvent ape) {

		String command = ape.getActionCommand();

		if (command.equals("save")) { // saves current content in current file
			String saveError = lockFrame.saveCurrentContent();
			if (saveError != null) {
				PeaDialog.showMessage(lockFrame, saveError, PeaProperties.getVmBridge().translate("error"), 0);
			}

		} else if (command.equals("export")) {

			Object[] possibleOptions = { PeaProperties.getVmBridge().translate("simple_plaintext_file"), // "A. save as

					PeaProperties.getVmBridge().translate("plaintext_file_rtf") };// "B. save as plain text in rich text
			// TODO HTML																	// format (unencrypted)"};
			Object selectedValue = JOptionPane.showInputDialog(this,
					PeaProperties.getVmBridge().translate("choose_how_to_save"), // "Choose how you want to save the file:
																				// \n",
					PeaProperties.getVmBridge().translate("choose"), // "Choose...",
					JOptionPane.PLAIN_MESSAGE, null, possibleOptions, possibleOptions[0]);
			FileFilter filter = null;
			String fileName = null;
			String filePath = null;

			if (selectedValue == null) { // cancel
				return;
			} else if (((String) selectedValue).startsWith("A")) {
				filter = new TxtFilter();
			} else if (((String) selectedValue).startsWith("B")) {
				filter = new RtfFilter();
			}
			final PeaFileChooser fc = new PeaFileChooser(this);
			fc.setFileFilter(filter);
			int returnVal = fc.showSaveDialog((JFrame) PeaControl.getDialog().getLockFrame());
			if (returnVal == PeaFileChooser.APPROVE_OPTION) {
				fileName = fc.getSelectedFile().getName();
				filePath = fc.getSelectedFile().getPath();

			} else { // cancel option
				return;
			}

			// WriteRessources...
			if (filter instanceof TxtFilter) {
				String text = null;
				try {
					JTextPane textPane = LockFrameEditor.getFocusedTextPane();
					text = textPane.getDocument().getText(0, textPane.getDocument().getLength());
				} catch (BadLocationException e) {
					TestLog.e(LockFrameEditorMenu.class, "EditorLockFrame Menu (Save as txt): \n" + e.toString() + ", " + e.getMessage());
					JOptionPane.showMessageDialog(this,
							// PeaControl.getDialog().getLockFrame(),
							PeaProperties.getVmBridge().translate("unexpected_error"),
							PeaProperties.getVmBridge().translate("error"), JOptionPane.ERROR_MESSAGE);
				}
				if (text == null) {
					text = "";
				}
				byte[] textBytes = text.getBytes(PeaProperties.getCharset());// .Converter.chars2bytes(text.toCharArray());
				if (!fileName.endsWith(".txt")) {
					filePath = filePath + ".txt";
				}
				if (new File(filePath).exists()) {
					TestLog.v(this.getClass(), "Warning existing file will be overwritten");
					int n = JOptionPane.showConfirmDialog(this,
							// PeaControl.getDialog().getLockFrame(),
							PeaProperties.getVmBridge().translate("overwrite_existing_file"),
							PeaProperties.getVmBridge().translate("warning"), JOptionPane.OK_CANCEL_OPTION,
							JOptionPane.WARNING_MESSAGE);
					if (n != JFileChooser.APPROVE_OPTION) { // cancel, close
						return;
						// System.exit(0);
					}
				}
				boolean success = WriteResources.write(textBytes, filePath, null);
				if (success == false) {
					TestLog.e(LockFrameEditorMenu.class, "EditorLockFrame Menu (Save as txt): \n");
					JOptionPane.showMessageDialog(this,
							// PeaControl.getDialog().getLockFrame(),
							PeaProperties.getVmBridge().translate("read_write_failed"),
							PeaProperties.getVmBridge().translate("error"), JOptionPane.ERROR_MESSAGE);

				}
			} else if (filter instanceof RtfFilter) {

				if (!filePath.toLowerCase().endsWith("rtf")) {
					filePath = filePath + ".rtf";
				}
				if (new File(filePath).exists()) {
					TestLog.v(this.getClass(), "Warning existing file will be overwritten");
					int n = JOptionPane.showConfirmDialog(this,
							// PeaControl.getDialog().getLockFrame(),
							PeaProperties.getVmBridge().translate("overwrite_existing_file"),
							PeaProperties.getVmBridge().translate("warning"), JOptionPane.OK_CANCEL_OPTION,
							JOptionPane.WARNING_MESSAGE);
					if (n != JFileChooser.APPROVE_OPTION) { // cancel, close
						return;
						// System.exit(0);
					}
				}
				JTextPane textPane = LockFrameEditor.getFocusedTextPane();
				StyledDocument doc = textPane.getStyledDocument();
				RTFEditorKit kit = new RTFEditorKit();
				BufferedOutputStream out;
				try {
					out = new BufferedOutputStream(new FileOutputStream(filePath));
					kit.write(out, doc, doc.getStartPosition().getOffset(), doc.getLength());
					out.close();
				} catch (FileNotFoundException e) {
					TestLog.e(LockFrameEditorMenu.class, "EditorLockFrame Menu (Save rtf): \n" + e.toString() + ", " + e.getMessage());
					JOptionPane.showMessageDialog(this,
							// PeaControl.getDialog().getLockFrame(),
							PeaProperties.getVmBridge().translate("read_write_failed") + "\n"
									+ PeaProperties.getVmBridge().translate("file_not_found") + ": \n" + filePath,
							PeaProperties.getVmBridge().translate("error"), JOptionPane.ERROR_MESSAGE);
				} catch (IOException e) {
					TestLog.e(LockFrameEditorMenu.class, "EditorLockFrame Menu (Save rtf): \n" + e.toString() + ", " + e.getMessage());
					JOptionPane.showMessageDialog(this,
							// PeaControl.getDialog().getLockFrame(),
							PeaProperties.getVmBridge().translate("read_write_failed") + "\n"
									+ PeaProperties.getVmBridge().translate("unexpected_error") + "\n " + e.getMessage()
									+ ": \n" + filePath,
							PeaProperties.getVmBridge().translate("error"), JOptionPane.ERROR_MESSAGE);
				} catch (BadLocationException e) {
					TestLog.e(LockFrameEditorMenu.class, "EditorLockFrame Menu (Save rtf): \n" + e.toString() + ", " + e.getMessage());
					JOptionPane.showMessageDialog(this,
							// PeaControl.getDialog().getLockFrame(),
							PeaProperties.getVmBridge().translate("read_write_failed") + "\n"
									+ PeaProperties.getVmBridge().translate("unexpected_error") + "\n " + e.getMessage()
									+ ": \n" + filePath,
							PeaProperties.getVmBridge().translate("error"), JOptionPane.ERROR_MESSAGE);
				}
			}
/* TODO			
		} else if (command.equals("dropboxStore")) {
		
			String fileName = "";
			try {
				byte[] plainBytes = ((LockFrameEditor) PeaControl.getDialog().getLockFrame()).getPlainBytes();
				fileName = LockFrameEditor.getCurrentTab().getFileName();

				ContentUtilities.saveContent(null, plainBytes, fileName);// gPswDialogBase.getEncryptedFileName());

				LockFrameEditor.setDocChangeUnsaved(false);
			} catch (Exception e) {
				TestLog.e(LockFrameEditorMenu.class, "EditorLockFrame Menu (save in upload to Dropbox): \n" + e.toString() + ", " + e.getMessage());
				PeaDialog.showMessage(lockFrame, PeaProperties.getOsBridge().translate("unexpected_error"),
						PeaProperties.getOsBridge().translate("error"), 0 // ERROR_MESSAGE
						);
			}
			
			if (! fileName.equals("")) {
				File[] files = { new File(fileName)};
				StorageProvider provider = null;
				try {
					provider = new DropboxProvider(PeaControl.getDialog().getPeaName(), this);
				} catch (java.lang.NoClassDefFoundError e) {
					PeaDialog.showMessage(this, PeaProperties.getOsBridge().translate("required_libraries_not_found"), "Error", 0);
					return;
				}
				boolean authSuccess = provider.authorize();
				if (authSuccess == false) {
					PeaDialog.showMessage(this, 
							PeaProperties.getOsBridge().translate("cloud_access_failed") +"\n"
							+ provider.getErrorMessages(), 
							PeaProperties.getOsBridge().translate("error"), 0);
				}
				boolean success = provider.uploadFiles(files, null);
				if (success == false) {
					PeaDialog.showMessage(this, 
							PeaProperties.getOsBridge().translate("cloud_access_failed") +"\n"
							+ provider.getErrorMessages(), 
							PeaProperties.getOsBridge().translate("error"), 0);
				}
			}
			*/

		} else if (command.equals("open")) { // current content get lost if not saved

			Object[] possibleOptions = { // "open encrypted lock file",
					PeaProperties.getVmBridge().translate("open_plaintext_file"),
					PeaProperties.getVmBridge().translate("open_plaintext_rtf_file"),
					// PeaProperties.getOsBridge().translate("open_html_file")
			};
			Object selectedValue = JOptionPane.showInputDialog(this,
					PeaProperties.getVmBridge().translate("open_what_kind_of_file"), // "Open what kind of file? \n",
					null, JOptionPane.PLAIN_MESSAGE, null, possibleOptions, possibleOptions[0]);
			FileFilter filter = null;
			File file = null;
			String fileName = null;
			if (selectedValue == null) { // cancel
				return;
			} else if (selectedValue.equals(possibleOptions[0])) {
				filter = new TxtFilter();
			} else if (selectedValue.equals(possibleOptions[1])) {
				filter = new RtfFilter();
			} /*
				 * else if ( selectedValue.equals(possibleOptions[2]) ) { filter = new
				 * FileNameExtensionFilter("Html files", "html", "HTML"); }
				 */
			final PeaFileChooser fc = new PeaFileChooser(this);
			fc.setFileFilter(filter);
			int returnVal = fc.showOpenDialog(this);
			// PeaControl.getDialog().getLockFrame());
			if (returnVal == PeaFileChooser.APPROVE_OPTION) {
				file = fc.getSelectedFile();
				try {
					fileName = file.getCanonicalPath();
				} catch (IOException e) {
					TestLog.e(LockFrameEditorMenu.class, "EditorLockFrame Menu (Open canonicalpath): \n" + e.toString() + ", " + e.getMessage());
					JOptionPane.showMessageDialog(this,
							// PeaControl.getDialog().getLockFrame(),
							PeaProperties.getVmBridge().translate("unexpected_error"),
							PeaProperties.getVmBridge().translate("error"), JOptionPane.ERROR_MESSAGE);
				}
			} else { // cancel option
				return;
			}
			try {
				// check if file is encrypted:
				int fileIdLen = Attachments.getFileIdentifierSize();
				byte[] end = null;
				try {
					end = Attachments.getEndBytesOfFile( 
							fileName, 
							fileIdLen);			
				} catch (IOException e) {
					TestLog.e(getClass(), new ExceptionHelper(e).getInfos() + " - " + fileName);
					PeaDialog.showMessage(this, 
							new ExceptionHelper(e).getInfos() + " - " + fileName,
							PeaProperties.getVmBridge().translate("error"), 0);
					return;
				} catch (Exception e) {
					TestLog.e(getClass(), new ExceptionHelper(e).getInfos() + " - " + fileName);
					PeaDialog.showMessage(this, 
							new ExceptionHelper(e).getInfos() + " - " + fileName,
							PeaProperties.getVmBridge().translate("error"), 0);
					return;
				}
				// check file identifier:
				byte[] fileId = new byte[fileIdLen];
				System.arraycopy(end, end.length - fileIdLen, fileId, 0, fileIdLen);
				if (  Arrays.equals(fileId, Attachments.getFileIdentifier())) {
					PeaDialog.showMessage(this, 
							PeaProperties.getVmBridge().translate("encrypted_file") + ": " + fileName,
							PeaProperties.getVmBridge().translate("error"), 0);
					return;
				}
				
				// warning: long execution time
				if (file.length() > 1048576) { // 1 MiB
					JOptionPane.showMessageDialog(this,
							// PeaControl.getDialog().getLockFrame(),
							"Large file may cause long execution time. ",
							PeaProperties.getVmBridge().translate("warning"), JOptionPane.WARNING_MESSAGE);
					// set the wait cursor:
					if (lockFrame == null) {
						lockFrame = (LockFrameEditor) PeaControl.getDialog().getLockFrame();
					}
					lockFrame.setCursor(CursorManager.getWaitCursor());
				}
				// avoid too large files to encrypt
				if (file.length() > 8388608) { // 8 MiB limit to avoid memory errors
					JOptionPane.showMessageDialog(this,
							// PeaControl.getDialog().getLockFrame(),
							"The selected file is too large. ", PeaProperties.getVmBridge().translate("error"),
							JOptionPane.ERROR_MESSAGE);
					// reset the cursor:
					if (lockFrame == null) {
						lockFrame = (LockFrameEditor) PeaControl.getDialog().getLockFrame();
					}
					lockFrame.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
					return;
				}
				JTextPane textPane = LockFrameEditor.getFocusedTextPane();
				Document doc = textPane.getDocument();
				int docLen = doc.getLength();

				if (fileName.endsWith(".txt") || fileName.endsWith(".TXT")) {
					byte[] textBytes = ReadResources.readExternFile(fileName);
					if (textBytes != null) {
						char[] textChars = Converter.bytes2chars(textBytes);

						try {
							doc.insertString(docLen, new String(textChars), null);
							lockFrame.setDocChangeUnsaved(true);
						} catch (BadLocationException e) {
							TestLog.e(getClass(), 
									"EditorLockFrame Menu (Open txt): \n" + e.toString() + ", " + e.getMessage());
							JOptionPane.showMessageDialog(this,
									// PeaControl.getDialog().getLockFrame(),
									PeaProperties.getVmBridge().translate("unexpected_error") + "\n " + e.getMessage()
											+ ": \n" + fileName,
									PeaProperties.getVmBridge().translate("error"), JOptionPane.ERROR_MESSAGE);
						}
						Zeroizer.zero(textChars);
					}
				} else if (fileName.endsWith(".rtf") || fileName.endsWith(".RTF")) {

					RTFEditorKit kit = (RTFEditorKit) textPane.getEditorKit();
					
					// check if file contains OLE {\object
					 byte[] encoded = null;
					try {
						encoded = Files.readAllBytes(file.toPath());
					} catch (IOException e1) {
						TestLog.ve(getClass(), e1.toString() + ": " + e1.getLocalizedMessage());
						PeaDialog.showMessage(this, e1.getLocalizedMessage(), 
								PeaProperties.getVmBridge().translate("error"), 0);
						return;
					}
					if (encoded == null) {
						new UnexpectedValueException("byte[]", "encoded", "is null").printDescription();
						return;
					}
					 String content = new String(encoded, StandardCharsets.US_ASCII);//PeaProperties.getCharset()); 
					 int index = content.indexOf("{\\object");
					 if (index != -1) {
						 // Warning: 
						 int res = PeaDialog.showQuestion(this,
								 "This file contains the object linking and embedding (OLE) capability"
								 + "which allows embedding of malicious code. \n"
								 + "Do you want to open them anyway?", 
								 PeaProperties.getVmBridge().translate("warning"), -1);
						 if (res != 0) {
							 return;
						 }
					 }
					 

					FileInputStream in;
					try {
						in = new FileInputStream(fileName);
						kit.read(in, doc, docLen);
						in.close();
						lockFrame.setDocChangeUnsaved(true);
					} catch (FileNotFoundException e) {
						System.err
								.println("EditorLockFrame Menu (Open rtf): \n" + e.toString() + ", " + e.getMessage());
						JOptionPane.showMessageDialog(this,
								// PeaControl.getDialog().getLockFrame(),
								PeaProperties.getVmBridge().translate("file_not_found") + "\n " + e.getMessage() + ": \n"
										+ fileName,
								PeaProperties.getVmBridge().translate("error"), JOptionPane.ERROR_MESSAGE);
					} catch (IOException e) {
						System.err
								.println("EditorLockFrame Menu (Open rtf): \n" + e.toString() + ", " + e.getMessage());
						JOptionPane.showMessageDialog(this,
								// PeaControl.getDialog().getLockFrame(),
								PeaProperties.getVmBridge().translate("unsuitable_rtf_file") + "\n " + e.getMessage()
										+ ": \n" + fileName,
								PeaProperties.getVmBridge().translate("error"), JOptionPane.ERROR_MESSAGE);
					} catch (BadLocationException e) {
						System.err
								.println("EditorLockFrame Menu (Open rtf): \n" + e.toString() + ", " + e.getMessage());
						JOptionPane.showMessageDialog(this,
								// PeaControl.getDialog().getLockFrame(),
								PeaProperties.getVmBridge().translate("unexpected_error") + "\n " + e.getMessage()
										+ ": \n" + fileName,
								PeaProperties.getVmBridge().translate("error"), JOptionPane.ERROR_MESSAGE);
					}
				} else {
					String[] options = {PeaProperties.getVmBridge().translate("cancel"), 
							PeaProperties.getVmBridge().translate("ignore_and_go_on")};
					int res = PeaDialog.showOptions(this, 
							PeaProperties.getVmBridge().translate("unsuitable_format") + ": "
							+ fileName, 
							PeaProperties.getVmBridge().translate("warning"), 
							0, 2, options, 0);
					
					if (res == 1) {

						byte[] textBytes = ReadResources.readExternFile(fileName);
						if (textBytes != null) {
							char[] textChars = Converter.bytes2chars(textBytes);

							try {
								doc.insertString(docLen, new String(textChars), null);
								lockFrame.setDocChangeUnsaved(true);
							} catch (BadLocationException e) {
								TestLog.e(getClass(), 
										"EditorLockFrame Menu (Open txt): \n" + e.toString() + ", " + e.getMessage());
								JOptionPane.showMessageDialog(this,
										// PeaControl.getDialog().getLockFrame(),
										PeaProperties.getVmBridge().translate("unexpected_error") + "\n " + e.getMessage()
										+ ": \n" + fileName,
										PeaProperties.getVmBridge().translate("error"), JOptionPane.ERROR_MESSAGE);
							}
							Zeroizer.zero(textChars);
						}
					} else {
						return;
					}
				}
				
				
				/*
					 * else if (fileName.endsWith(".html") || fileName.endsWith(".HTML")) {
					 * convertToRTF(fileName); }
					 */
			
			} finally {
				// reset the cursor:
				if (lockFrame == null) {
					lockFrame = (LockFrameEditor) PeaControl.getDialog().getLockFrame();
				}
				lockFrame.setCursor(CursorManager.getDefaultCursor());
			}

			
		} else if (command.equals("fontColor")) {

			Color color = JColorChooser.showDialog(this,
					// PeaControl.getDialog().getLockFrame(),
					PeaProperties.getVmBridge().translate("font_color"), Color.BLACK);

			if (color != null) {
				// TestLog.v(this.getClass(), color);
				// set color
				SimpleAttributeSet attr = new SimpleAttributeSet();
				StyleConstants.setForeground(attr, color);
				JTextPane textPane = LockFrameEditor.getFocusedTextPane();
				textPane.setCharacterAttributes(attr, false);
				//LockFrameEditor.setDocChangeUnsaved(true);
			}

		} else if (command.equals("randomPsw")) {
			JDialog dialog = PeaSettings.getPswGenerator();
			if (dialog == null) {
				if (PeaProperties.isTestMode()) {
					new UnexpectedValueException("dialog", "JDialog", "is null").printDescription();
				}
				return;
			}
			dialog.setIconImage(PswDialogView.getImage());
			dialog.setLocationRelativeTo(this);
			dialog.setVisible(true);
			// PeaSettings.getPswGenerator().setVisible(true);

		} else if (command.equals("psw")) {
			// save unsaved changes:
			LockFrameEditor lfe = null;
			Object obj =	PeaControl.getDialog().getLockFrame();
			if (obj instanceof LockFrameEditor) {
				lfe = (LockFrameEditor) obj;
			} else {
				new UnexpectedValueException("LockFrameEditor", "lfe", "is invalid instance").printDescription();
				return;
			}
			
			HashSet<String> fileNamesSet = LockFrameEditor.getCurrentTab().getFileNames();
		//	String[] fileNamesArray = Converter.hashSetToArray(fileNamesSet);
			boolean cloudFileIncluded = false; 
			if (fileNamesSet != null && ! fileNamesSet.isEmpty()) {
				for (String s : fileNamesSet) {
					if (s.contains(CloudControl.getTmpDir())) {
						cloudFileIncluded = true;
						break;
					}
				}
			} else {
				new UnexpectedValueException("HashSet<String>", "fileNamesSét", "is null").printDescription();
				return;
			}
			
			if (lfe.getNumberOfTabs() > 1) {
				// only for current tab!
				// confirm warning: other tab cannot be opened
				String[] options = {PeaProperties.getVmBridge().translate("cancel"), 
						PeaProperties.getVmBridge().translate("ok")};
				String cloudWarning = "";
				if ( cloudFileIncluded == true) {
					cloudWarning = "\n" + PeaProperties.getVmBridge().translate("cloud_access_warning");
				}
				// Warning: The change only applies to this file,
				// 	other files cannot be opened afterwards in this session.
				int opt = PeaDialog.showOptions(this, 
						PeaProperties.getVmBridge().translate("change_apply_this_tab") + cloudWarning, 
						PeaProperties.getVmBridge().translate("warning"), 0, 2, options, 0);
				if (opt != 1) {
					return;
				}
			}
			if (lfe.checkUnsavedContentChanges() == true) {
				lockFrame.setDocChangeUnsaved(false);
			}

			String changeError = lfe.changePasswordOrApplyChanges(lfe, false);
			if (changeError != null) {
				PeaDialog.showMessage(lfe, changeError, 
						PeaProperties.getVmBridge().translate("error"), 0);
			} else {
				TestLog.o(getClass(), "Successfully changed password/algorithms for current file "
						+ lfe.getCurrentFileNames()[0]);
//TestLog.v(getClass(), "Used scheme: " + AlgoParamHandler.writeString(false).getBytes(AttachmentHandler.getASCIICharset()));
				lfe.setChangesApplied(true);
			}

		} else if (command.equals("quit")) {
			try {
				//((LockFrameEditor) PeaControl.getDialog().getLockFrame()).checkUnsavedTextChanges();
				((LockFrameEditor) PeaControl.getDialog().getLockFrame()).windowClosingCommands();

				JTextPane textPane = LockFrameEditor.getFocusedTextPane();
				StyledDocument doc = textPane.getStyledDocument();
				try {
					doc.remove(0, doc.getLength());
				} catch (BadLocationException e) {
					// don't show message...
					TestLog.e(LockFrameEditorMenu.class, "LockFrameEditorMenu (Quit): \n" + "Can't remove text from Textpane, \n"
							+ e.getMessage() + ", " + e.toString());
				}
				System.exit(0);
			} catch (Exception e) {
				int result = PeaDialog.showQuestion(this, e.getLocalizedMessage() 
						+ "\n   " + PeaProperties.getVmBridge().translate("quit") + "?", 
						PeaProperties.getVmBridge().translate("error"), 0);
				if (result == 0) {
					System.exit(0);
				}
			}
		} else if (command.equals("undo")) {
			if (LockFrameEditor.manager.canUndo()) {
				LockFrameEditor.manager.undo();
			}
		} else if (command.equals("redo")) {
			if (LockFrameEditor.manager.canRedo()) {
				LockFrameEditor.manager.redo();
			}
			
		} else if (command.equals("search")) {
			// open input dialog
			String searchText = PeaDialog.showInputDialog(this, 
					PeaProperties.getVmBridge().translate("search_for"), 
					PeaProperties.getVmBridge().translate("search"), -1);
			if (searchText == null || searchText.length() == 0) {
				return;
			}
			int searchTextLen = searchText.length();
			//TestLog.v(this.getClass(), "search for " + searchText);

			JTextPane textPane = LockFrameEditor.getFocusedTextPane();
			// remove highlighter from previous search
			textPane.getHighlighter().removeAllHighlights();
			Document doc = textPane.getDocument();
			
			String tabText = null;
			int docLen = doc.getLength();
			if (docLen == 0) {
				TestLog.e(LockFrameEditorMenu.class, "No text to search in...");
				return;
			}
			try {
				tabText = doc.getText(0, docLen);
			} catch (BadLocationException e1) {
				TestLog.e(getClass(), new ExceptionHelper(e1).getInfos());
				//e1.printStackTrace();
				return;
			}
			

			int pos= 0;
			int endPos = 0;
			DefaultHighlighter.DefaultHighlightPainter highlightPainter = 
			        new DefaultHighlighter.DefaultHighlightPainter(Color.YELLOW);
			// Note: The hightlight is removes by mouse click in TabComponent MouseListener
			while (endPos < docLen) {
				pos = tabText.indexOf(searchText, endPos);
				if (pos == -1) {
					break;
				}
				endPos = pos + searchTextLen;
				// highlight
				try {
					textPane.getHighlighter().addHighlight(pos, endPos, 
							highlightPainter);
				} catch (BadLocationException e) {
					TestLog.e(getClass(), new ExceptionHelper(e).getInfos());
					TestLog.e(LockFrameEditorMenu.class, "While searching for " + searchText);
				}
			}
			
		} else if (command.equals("cut")) {
			JTextPane textPane = LockFrameEditor.getFocusedTextPane();
			textPane.cut();
		} else if (command.equals("copy")) {
			JTextPane textPane = LockFrameEditor.getFocusedTextPane();
			textPane.copy();
		} else if (command.equals("paste")) {
			JTextPane textPane = LockFrameEditor.getFocusedTextPane();
			textPane.paste();			
			// Clipboard clipboard = Toolkit.getDefaultToolkit().getSystemClipboard();
			// TransferHandler transferHandler =
			// LockFrameCalendar.textPane.getTransferHandler();
			// transferHandler.importData(LockFrameCalendar.textPane,
			// clipboard.getContents(null));
		} else if (command.equals("selectAll")) {
			JTextPane textPane = LockFrameEditor.getFocusedTextPane();
			textPane.selectAll();
			
		} else if (command.startsWith("defaultFontFamily")) {
			String family = command.substring(17, command.length());
			if (family.equals("SERIF")) {
				NotebookSetting.setFontFamily(Font.SERIF);
			} else 	if (family.equals("SANS_SERIF")) {
				NotebookSetting.setFontFamily(Font.SANS_SERIF);
			} else 	if (family.equals("MONOSPACED")) {
				NotebookSetting.setFontFamily(Font.MONOSPACED);
			} else {
				new UnexpectedValueException("family", "String", "does not match font family constants").printDescription();
			}
			// Set the default settings: 
			MutableAttributeSet attributes = LockFrameEditor.getFocusedTextPane().getInputAttributes();
			StyleConstants.setFontFamily(attributes, NotebookSetting.getFontFamily());
	        // get the document
	        StyledDocument document = LockFrameEditor.getFocusedTextPane().getStyledDocument();
	        // Replace the style at end of document
	        document.setCharacterAttributes(document.getLength(), 1, attributes, true);
			
		} else if (command.startsWith("defaultFontSize")) {
			String sizeString = command.substring(15, command.length());
			int defaultSize = NotebookSetting.getFontSize();
			try {
				defaultSize = Integer.parseInt(sizeString);
			} catch (Exception e) {
				new UnexpectedValueException("sizeString", "String", "can not be parsed: " + sizeString).printDescription();
			}			
			NotebookSetting.setFontSize(defaultSize);
			// Set the default settings: 
			MutableAttributeSet attributes = LockFrameEditor.getFocusedTextPane().getInputAttributes();;
	        StyleConstants.setFontSize(attributes, NotebookSetting.getFontSize()); 
	        StyledDocument document = LockFrameEditor.getFocusedTextPane().getStyledDocument();
	        // Replace the style at end of document
	        document.setCharacterAttributes(document.getLength(), 1, attributes, true);
		} else if (command.equals("defaultFontColor")) {

			Color color = JColorChooser.showDialog(this,
			PeaProperties.getVmBridge().translate("default_font_color"), Color.BLACK);

			if (color != null) {
				NotebookSetting.setFontColor(color);
				MutableAttributeSet attributes = LockFrameEditor.getFocusedTextPane().getInputAttributes();
		        StyleConstants.setForeground(attributes, NotebookSetting.getFontColor());
		        // get the document
		        StyledDocument document = LockFrameEditor.getFocusedTextPane().getStyledDocument();
		        // Replace the style at end of document
		        document.setCharacterAttributes(document.getLength(), 1, attributes, true);
			}
		} else if (command.equals("defaultBackground")) {
			Color color = JColorChooser.showDialog(this,
					// PeaControl.getDialog().getLockFrame(),
					PeaProperties.getVmBridge().translate("default_background"), NotebookSetting.getTextBackground());

			if (color != null) {
				NotebookSetting.setTextBackground(color);

				UIManager.put("TextPane[Enabled].backgroundPainter", LockFrameEditor.getCurrentTab().new BackgroundPainter() );// {
		/*		    @Override
				    public void paint(Graphics2D g, JComponent comp, int width, int height) {
				        g.setColor(NotebookSetting.getTextBackground());
				        g.fillRect(0, 0, width, height);
				    }
				}); */
				UIDefaults defaults = new UIDefaults();
				JTextPane textPane = LockFrameEditor.getFocusedTextPane();
				textPane.putClientProperty("Nimbus.Overrides", defaults);
				textPane.putClientProperty("Nimbus.Overrides.InheritDefaults", true);
				textPane.setBackground(NotebookSetting.getTextBackground());
			}
		} else {
			TestLog.e(LockFrameEditorMenu.class, "invalid command in editor menu: " + command);
		}
	}
	/*
	 * This can only replace the existing content and only works for html 3...
	 * 
	 * private void convertToRTF(String fileName) {
	 * 
	 * String htmlStr = new String( ReadResources.readExternFile(fileName),
	 * PswDialogBase.getCharset());
	 * 
	 * OutputStream os = new ByteArrayOutputStream(); HTMLEditorKit htmlEditorKit =
	 * new HTMLEditorKit(); RTFEditorKit rtfEditorKit = new RTFEditorKit(); String
	 * rtfStr = null;
	 * 
	 * htmlStr = htmlStr.replaceAll("<br.*?>","#NEW_LINE#"); htmlStr =
	 * htmlStr.replaceAll("</p>","#NEW_LINE#"); htmlStr =
	 * htmlStr.replaceAll("</li>","#NEW_LINE#"); htmlStr =
	 * htmlStr.replaceAll("<p.*?>",""); InputStream is = new
	 * ByteArrayInputStream(htmlStr.getBytes()); try { Document doc =
	 * htmlEditorKit.createDefaultDocument(); htmlEditorKit.read(is, doc, 0);
	 * rtfEditorKit .write(os, doc, 0, doc.getLength()); rtfStr = os.toString();
	 * rtfStr = rtfStr.replaceAll("#NEW_LINE#","\\\\par ");
	 * LockFrameCalendar.textPane.setText( rtfStr); is.close(); } catch (IOException
	 * e) { e.printStackTrace(); } catch (BadLocationException e) {
	 * e.printStackTrace(); } }
	 */

	// --------------------------------
	// inner classes
	//
	private class TxtFilter extends FileFilter {
		public boolean accept(File f) {
			return f.isDirectory() || f.getName().toLowerCase().endsWith(".txt")
					|| f.getName().toLowerCase().endsWith(".text");
		}

		public String getDescription() {
			return "text" + " (*.txt)";
		}
	};

	private class RtfFilter extends FileFilter {
		public boolean accept(File f) {
			return f.isDirectory() || f.getName().toLowerCase().endsWith(".rtf");
		}

		public String getDescription() {
			return "rich text format" + " (*.rtf)";
		}
	};
	
	protected void setRootWindowForCloudMenu(Window win) {
		cloudMenu.setRooTWindow(win);
	}
	protected static void setLockFrame(LockFrameEditor lfe) {
		if (lfe != null) {
			lockFrame = lfe;
		}
	}

    public AccessibleContext getAccessibleContext() {
        if (accessibleContext == null) {
            accessibleContext = new AccessibleLockFrameEditorMenu();
        }
        return accessibleContext;
    }

    private class AccessibleLockFrameEditorMenu extends AccessiblePeaMenuBar {
        //Inherit everything, override nothing.
    }
}